@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2023-2025 GraphicsMagick Group
+% Copyright (C) 2023-2026 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -28,6 +28,18 @@
 * Meanwhile, 1.14.0 is required to compile.
 *
 * Nokia technical publication on HEIF: https://nokiatech.github.io/heif/technical.html
+*
+* Yet to be done:
+*
+*   o Read/render HEIF tiles directly.
+*   o Possibly support reading directly from file (heif_context_read_from_file())
+*     rather than an in-memory BLOB.
+*   o Support selecting to read HEIF depth image.
+*   o Use HEIF transform properties to evaluate image orientation (requires 1.17).
+*   o Provide user control over chroma upsampling options.
+*   o Support reading sequences (requires 1.20).
+*
+*   o Write HEIF (HEIC, AVIF, more?)
 */
 
 #include "magick/studio.h"
@@ -50,6 +62,11 @@
 /* Set to 1 to enable the currently non-functional progress monitor callbacks */
 #define HEIF_ENABLE_PROGRESS_MONITOR 0
 
+#if defined(HAVE_LIBHEIF_HEIF_SEQUENCES_H)
+/* Set to 1 to enable use of <libheif/heif_sequences.h> APIs */
+#define HEIF_SUPPORT_SEQUENCES 1
+#endif /* if defined(HAVE_LIBHEIF_HEIF_SEQUENCES_H) */
+
 /*
   Libheif added LIBHEIF_MAKE_VERSION in 1.14.0 (October 31, 2021).
   Duplicate its definition so we can detect versions lower than
@@ -120,6 +137,9 @@
   }
 
 #include <libheif/heif.h>
+#if defined(HEIF_SUPPORT_SEQUENCES) && HEIF_SUPPORT_SEQUENCES
+#include <libheif/heif_sequences.h>
+#endif /* if defined(HEIF_SUPPORT_SEQUENCES) */
 static MagickBool heif_initialized = MagickFalse;
 
 /*
@@ -156,25 +176,46 @@ static MagickBool IsHEIF(const unsigned char *magick,const size_t length)
   enum heif_filetype_result
     heif_filetype;
 
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "Testing header for supported HEIF format");
+  MagickBool
+    status = MagickFalse;
 
-  /*
-    const char* heif_get_file_mime_type(const uint8_t* data, int len)
-    heif_filetype_result heif_check_filetype(const uint8_t* data, int len)
-  */
-
-  if (length < 12)
-    return(MagickFalse);
+  if ((length < 12) || (length > (size_t) INT_MAX))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Data is too short (%zu bytes)", length);
+      return status;
+    }
 
   heif_filetype = heif_check_filetype(magick, (int) length);
-  if (heif_filetype == heif_filetype_yes_supported)
-    return MagickTrue;
-
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "Not a supported HEIF format");
-
-  return(MagickFalse);
+  switch (heif_filetype)
+    {
+    case heif_filetype_yes_supported:
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Data is HEIF (supported by libheif)");
+      status = MagickTrue;
+      break;
+    case heif_filetype_yes_unsupported:
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Data is HEIF (unsupported by libheif)");
+      status = MagickTrue;
+      break;
+    case heif_filetype_maybe:
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Data may be HEIF");
+      status = MagickTrue;
+      break;
+    case heif_filetype_no:
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Data not HEIF");
+      status = MagickFalse;
+      break;
+    default:
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Unexpected enum value %d",
+                            (int) heif_filetype);
+      status = MagickFalse;
+    }
+  return status;
 }
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -207,6 +248,7 @@ static MagickBool IsHEIF(const unsigned char *magick,const size_t length)
 #define HEIFReadCleanup()                                              \
   do                                                                   \
     {                                                                  \
+      MagickFreeResourceLimitedMemory(heif_item_id *,item_ids);        \
       if (heif_image)                                                  \
         heif_image_release(heif_image);                                \
       if (heif_image_handle)                                           \
@@ -215,7 +257,8 @@ static MagickBool IsHEIF(const unsigned char *magick,const size_t length)
         heif_context_free(heif);                                       \
       if (decode_options)                                              \
         heif_decoding_options_free(decode_options);                    \
-      MagickFreeResourceLimitedMemory(unsigned char *,in_buf);         \
+      if (in_buf_allocated)                                            \
+        MagickFreeResourceLimitedMemory(unsigned char *,in_buf);       \
     } while (0);
 
 #define ThrowHEIFReaderException(code_,reason_,image_) \
@@ -1164,6 +1207,13 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
     }
 
   {
+
+    typedef union
+    {
+      const uint8_t *u8;
+      const uint16_t *u16;
+    } magick_plane_t;
+
     const char *
       heif_channel_str;
 
@@ -1187,11 +1237,11 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
           static const enum heif_channel channels_YCbCrA[4] =
             { heif_channel_Y, heif_channel_Cb, heif_channel_Cr, heif_channel_Alpha };
 
-          const uint8_t *
-            planes[4]  = { 0, 0, 0, 0 };
+          magick_plane_t
+            planes[4];
 
-          const uint8_t *
-            line[4] = { 0, 0, 0, 0 };
+          magick_plane_t
+            line[4];
 
           size_t
             row_stride[4];
@@ -1213,21 +1263,31 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
           for (channel_index = 0; channel_index < num_channels; channel_index++)
             {
               heif_channel = channels_YCbCrA[channel_index];
-              planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
-                                                                     heif_channel,
-                                                                     &row_stride[channel_index]);
               heif_channel_str = HEIF_channel_to_string(heif_channel);
+              planes[channel_index].u8 = heif_image_get_plane_readonly2(heif_image,
+                                                                        heif_channel,
+                                                                        &row_stride[channel_index]);
+              if (planes[channel_index].u8 == (const uint8_t*) NULL)
+                {
+                  if (image->logging)
+                    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                          "heif_image_get_plane_readonly2() returned NULL"
+                                          " for channel index %u (channel %s)!",
+                                          channel_index,heif_channel_str);
+                  ThrowHEIFThrowReadImageFrameException(CorruptImageError,
+                                                        AnErrorHasOccurredReadingFromFile, image);
+                }
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
               bits_per_pixel_range = heif_image_get_bits_per_pixel_range(heif_image,
                                                                          heif_channel);
               if ((int) image->depth < bits_per_pixel_range)
                 image->depth=(unsigned int) bits_per_pixel_range;
-              assert(planes[channel_index] != (const uint8_t*) NULL);
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index],
+                                      bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1244,7 +1304,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
+                line[channel_index].u8=planes[channel_index].u8+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1257,13 +1317,14 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
 
                       if (image->depth > 8)
                         {
-                          sample = *((uint16_t*) line[channel_index]);
-                          line[channel_index] += 2;
+                          sample = *line[channel_index].u16;
+                          line[channel_index].u16++;
                           quantum=ScaleAnyToQuantum(sample,max_value_given_bits);
                         }
                       else
                         {
-                          sample = *line[channel_index]++;
+                          sample = *line[channel_index].u8;
+                          line[channel_index].u8++;
                           quantum=ScaleCharToQuantum(sample);
                         }
                       switch (channel_index)
@@ -1306,11 +1367,11 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
             channels_RGBA[4] =
             { heif_channel_R, heif_channel_G, heif_channel_B, heif_channel_Alpha };
 
-          const uint8_t *
-            planes[4]  = { 0, 0, 0 , 0 };
+          magick_plane_t
+            planes[4];
 
-          const uint8_t *
-            line[4] = { 0, 0, 0 , 0 };
+          magick_plane_t
+            line[4];
 
           size_t
             row_stride[4];
@@ -1333,21 +1394,31 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
           for (channel_index = 0; channel_index < num_channels; channel_index++)
             {
               heif_channel = channels_RGBA[channel_index];
-              planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
-                                                                     heif_channel,
-                                                                     &row_stride[channel_index]);
               heif_channel_str = HEIF_channel_to_string(heif_channel);
+              planes[channel_index].u8 = heif_image_get_plane_readonly2(heif_image,
+                                                                        heif_channel,
+                                                                        &row_stride[channel_index]);
+              if (planes[channel_index].u8 == (const uint8_t*) NULL)
+                {
+                  if (image->logging)
+                    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                          "heif_image_get_plane_readonly2() returned NULL"
+                                          " for channel index %u (channel %s)!",
+                                          channel_index,heif_channel_str);
+                  ThrowHEIFThrowReadImageFrameException(CorruptImageError,
+                                                        AnErrorHasOccurredReadingFromFile, image);
+                }
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
               bits_per_pixel_range = heif_image_get_bits_per_pixel_range(heif_image,
                                                                          heif_channel);
               if ((int) image->depth < bits_per_pixel_range)
                 image->depth=(unsigned int) bits_per_pixel_range;
-              assert(planes[channel_index] != (const uint8_t*) NULL);
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index],
+                                      bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1364,7 +1435,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
+                line[channel_index].u8=planes[channel_index].u8+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1377,13 +1448,14 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
 
                       if (image->depth > 8)
                         {
-                          sample = *((uint16_t*) line[channel_index]);
-                          line[channel_index] += 2;
+                          sample = *line[channel_index].u16;
+                          line[channel_index].u16++;
                           quantum=ScaleAnyToQuantum(sample,max_value_given_bits);
                         }
                       else
                         {
-                          sample=*line[channel_index]++;
+                          sample = *line[channel_index].u8;
+                          line[channel_index].u8++;
                           quantum=ScaleCharToQuantum(sample);
                         }
                       switch (channel_index)
@@ -1426,11 +1498,11 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
           static const enum heif_channel channels_YA[4] =
             { heif_channel_Y, heif_channel_Alpha };
 
-          const uint8_t *
-            planes[2]  = { 0, 0 };
+          magick_plane_t
+            planes[2];
 
-          const uint8_t *
-            line[2] = { 0, 0 };
+          magick_plane_t
+            line[2];
 
           size_t
             row_stride[2];
@@ -1454,9 +1526,20 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
           for (channel_index = 0; channel_index < num_channels; channel_index++)
             {
               heif_channel = channels_YA[channel_index];
-              planes[channel_index] = heif_image_get_plane_readonly2(heif_image,
-                                                                     heif_channel,
-                                                                     &row_stride[channel_index]);
+              heif_channel_str = HEIF_channel_to_string(heif_channel);
+              planes[channel_index].u8 = heif_image_get_plane_readonly2(heif_image,
+                                                                        heif_channel,
+                                                                        &row_stride[channel_index]);
+              if (planes[channel_index].u8 == (const uint8_t*) NULL)
+                {
+                  if (image->logging)
+                    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                          "heif_image_get_plane_readonly2() returned NULL"
+                                          " for channel index %u (channel %s)!",
+                                          channel_index,heif_channel_str);
+                  ThrowHEIFThrowReadImageFrameException(CorruptImageError,
+                                                        AnErrorHasOccurredReadingFromFile, image);
+                }
               bits_per_pixel = heif_image_get_bits_per_pixel(heif_image,
                                                              heif_channel);
               bits_per_pixel_range = heif_image_get_bits_per_pixel_range(heif_image,
@@ -1464,11 +1547,11 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
               if ((int) image->depth < bits_per_pixel_range)
                 image->depth=(unsigned int) bits_per_pixel_range;
               heif_channel_str = HEIF_channel_to_string(heif_channel);
-              assert(planes[channel_index] != (const uint8_t*) NULL);
               if (image->logging)
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                       "  channel %s row stride %zu (%d bits in %d bit quantum)",
-                                      heif_channel_str, row_stride[channel_index], bits_per_pixel_range, bits_per_pixel);
+                                      heif_channel_str, row_stride[channel_index],
+                                      bits_per_pixel_range, bits_per_pixel);
             }
           max_value_given_bits=MaxValueGivenBits(image->depth);
 
@@ -1485,7 +1568,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                   return MagickFail;
                 }
               for (channel_index = 0; channel_index < num_channels; channel_index++)
-                line[channel_index]=planes[channel_index]+y*row_stride[channel_index];
+                line[channel_index].u8=planes[channel_index].u8+y*row_stride[channel_index];
               for (x=0; x < (long)image->columns; x++)
                 {
                   for (channel_index = 0; channel_index < num_channels; channel_index++)
@@ -1498,13 +1581,14 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
 
                       if (image->depth > 8)
                         {
-                          sample = *((uint16_t*) line[channel_index]);
-                          line[channel_index] += 2;
+                          sample = *line[channel_index].u16;
+                          line[channel_index].u16++;
                           quantum=ScaleAnyToQuantum(sample,max_value_given_bits);
                         }
                       else
                         {
-                          sample=*line[channel_index]++;
+                          sample = *line[channel_index].u8;
+                          line[channel_index].u8++;
                           quantum=ScaleCharToQuantum(sample);
                         }
 
@@ -1516,7 +1600,7 @@ static MagickPassFail ReadHEIFImageFrame(heif_image_handle* heif_image_handle,
                             SetOpacitySample(q,OpaqueOpacity);
                           break;
                         case 1:
-                          SetOpacitySample(q,MaxRGB-ScaleCharToQuantum(*line[channel_index]++));
+                          SetOpacitySample(q,MaxRGB-quantum);
                           break;
                         }
                     }
@@ -1574,17 +1658,38 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   unsigned char
     *in_buf = NULL;
 
+  heif_item_id
+    *item_ids = NULL;
+
+  const char *
+    magick;
+
   size_t
-    in_len;
+    in_len = 0;
 
   struct heif_error
     heif_status;
 
+  int
+#if defined(HEIF_SUPPORT_SEQUENCES) && HEIF_SUPPORT_SEQUENCES
+    has_sequence,
+#endif /* if defined(HEIF_SUPPORT_SEQUENCES) && HEIF_SUPPORT_SEQUENCES */
+    number_of_top_level_images;
+
+  char
+    fourcc[5];
+
+  heif_item_id
+    primary_image_id;
+
 #if HEIF_ENABLE_PROGRESS_MONITOR
   ProgressUserData
     progress_user_data;
 #endif
 
+  MagickBool
+    in_buf_allocated=MagickFalse;
+
   MagickPassFail
     status;
 
@@ -1593,6 +1698,8 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
 
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "ImageInfo magick = %s", image_info->magick);
   /*
     Open image file.
   */
@@ -1618,7 +1725,8 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
 
   decode_options->ignore_transformations = image_info->ping ? MagickTrue : MagickFalse;
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "Initial ignore_transformations = %s", decode_options->ignore_transformations ? "True" : "False");
+                        "Initial ignore_transformations = %s",
+                        decode_options->ignore_transformations ? "True" : "False");
 #if HEIF_ENABLE_PROGRESS_MONITOR
   progress_user_data.exception = exception;
   progress_user_data.image = image;
@@ -1641,22 +1749,35 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
 #endif /* if QuantumDepth == 8 */
 
 
-  /*
-    Read the whole input file into a memory blob
-  */
   in_len=GetBlobSize(image);
-  in_buf=MagickAllocateResourceLimitedArray(unsigned char *,in_len,sizeof(*in_buf));
+  in_buf=GetBlobStreamData(image);
   if (in_buf == (unsigned char *) NULL)
-    ThrowHEIFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    {
+      /*
+        Read the whole input file into a memory blob
+      */
+     in_buf=MagickAllocateResourceLimitedArray(unsigned char *,in_len,sizeof(*in_buf));
+      if (in_buf == (unsigned char *) NULL)
+        ThrowHEIFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+      in_buf_allocated=MagickTrue;
 
-  if (image->logging)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "Reading file into in-memory blob...");
-  if (ReadBlob(image,in_len,in_buf) != in_len)
-    ThrowHEIFReaderException(CorruptImageError, UnexpectedEndOfFile, image);
-  if (image->logging)
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "  Done!");
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Reading file into in-memory blob...");
+      if (ReadBlob(image,in_len,in_buf) != in_len)
+        ThrowHEIFReaderException(CorruptImageError, UnexpectedEndOfFile, image);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "  Done!");
+    }
+  else
+    {
+      /*
+        Accessing memory-mapped input file or blob memory directly.
+      */
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Input accessed as in-memory blob");
+    }
 
   /* Init HEIF-Decoder handles */
   heif=heif_context_alloc();
@@ -1664,14 +1785,34 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
   /* Apply security limits */
   (void) apply_security_limits(image_info,heif);
 
+  /* Header sanity check */
+  if (!IsHEIF(in_buf, in_len))
+    ThrowHEIFReaderException(CorruptImageError, ImproperImageHeader, image);
+
+  /* Test file header for format, and update image magick string */
+  magick="HEIF";
+  if (heif_has_compatible_brand(in_buf, in_len, "avci") == 1)
+    magick="AVCI";
+  else if (heif_has_compatible_brand(in_buf, in_len, "avif") == 1)
+    magick="AVIF";
+  else if (heif_has_compatible_brand(in_buf, in_len, "heix") == 1)
+    magick="HEIC";
+  else if (heif_has_compatible_brand(in_buf, in_len, "heic") == 1)
+    magick="HEIC";
+  else if (heif_has_compatible_brand(in_buf, in_len, "mp41") == 1)
+    magick="MP4";
+  else if (heif_has_compatible_brand(in_buf, in_len, "mp42") == 1)
+    magick="MP4";
+  (void) strcpy(image->magick,magick);
+   if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "Assigned magick: %s", magick);
+
   /*
     Trace some information about the file
   */
   if (image->logging)
     {
-      char
-        fourcc[5];
-
       {
         /* Log main brand */
         (void) memset(fourcc,0,sizeof(fourcc));
@@ -1764,33 +1905,131 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
       ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
     }
 
-  /* FIXME: no support for reading multiple images but should be
-     added, if multiple images will use primary image */
-  {
-    int number_of_top_level_images;
-    number_of_top_level_images=heif_context_get_number_of_top_level_images(heif);
-    if (image->logging)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                            "Number of top level images: %d (reading primary image only)",
-                            number_of_top_level_images);
-    if (number_of_top_level_images == 0)
-      ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+#if defined(HEIF_SUPPORT_SEQUENCES) && HEIF_SUPPORT_SEQUENCES
+  has_sequence = heif_context_has_sequence(heif);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Has sequence: %c", has_sequence ? 'Y' : 'N');
+#endif /* if defined(HEIF_SUPPORT_SEQUENCES) && HEIF_SUPPORT_SEQUENCES */
 
-    heif_status=heif_context_get_primary_image_handle(heif, &heif_image_handle);
-    if (heif_status.code == heif_error_Memory_allocation_error)
-      ThrowHEIFReaderException(ResourceLimitError, MemoryAllocationFailed, image);
-    if (heif_status.code != heif_error_Ok)
-      {
-        if (image->logging)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "heif_context_get_primary_image_handle() reports error \"%s\"",
-                                heif_status.message);
-        ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
-      }
-  }
+  /* Get the number of top level images */
+  number_of_top_level_images=heif_context_get_number_of_top_level_images(heif);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Number of top level images: %d",
+                          number_of_top_level_images);
+  if (number_of_top_level_images <= 0)
+    {
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "heif_context_get_number_of_top_level_images() reports %d \"%s\"",
+                              number_of_top_level_images, heif_status.message);
+      ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+    }
+
+  /* Get the primary image id */
+  heif_status=heif_context_get_primary_image_ID(heif,&primary_image_id);
+  if (heif_status.code != heif_error_Ok)
+    {
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "heif_context_get_primary_image_ID() reports error \"%s\"",
+                              heif_status.message);
+      ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+    }
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "Primary image id: %d", primary_image_id);
+
+  /* Get a handle for the primary image id */
+  heif_status=heif_context_get_image_handle(heif, primary_image_id, &heif_image_handle);
+  if (heif_status.code == heif_error_Memory_allocation_error)
+    ThrowHEIFReaderException(ResourceLimitError, MemoryAllocationFailed, image);
+  if (heif_status.code != heif_error_Ok)
+    {
+      if (image->logging)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "heif_context_get_image_handle() reports error \"%s\"",
+                              heif_status.message);
+      ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+    }
 
   status=ReadHEIFImageFrame(heif_image_handle,decode_options,image,image_info,exception);
 
+  heif_image_handle_release(heif_image_handle);
+  heif_image_handle=(struct heif_image_handle *) NULL;
+
+  /*
+    Read any additional top-level images
+  */
+  if ((status != MagickFail) && (number_of_top_level_images > 1))
+    {
+      int
+        item_num;
+
+      item_ids=MagickAllocateResourceLimitedArray(heif_item_id *,number_of_top_level_images,
+                                                  sizeof(*item_ids));
+      if (item_ids == (heif_item_id *) NULL)
+        ThrowHEIFReaderException(ResourceLimitError, MemoryAllocationFailed, image);
+
+      heif_context_get_list_of_top_level_image_IDs(heif,item_ids,number_of_top_level_images);
+
+      for (item_num=0; item_num < number_of_top_level_images; item_num++)
+        {
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Image index %d: image id %d",
+                                  item_num, item_ids[item_num]);
+
+          /* Skip the primary image we already read. Hopefully this
+             was also the first image in the top level image list. */
+          if (item_ids[item_num] == primary_image_id)
+            {
+              if (image->logging)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "Item %d matches primary image id %d (skipping)",
+                                      item_num, primary_image_id);
+              continue;
+            }
+
+          /*
+            Allocate next image structure.
+          */
+          AllocateNextImage(image_info,image);
+          image=SyncNextImageInList(image);
+          (void) strlcpy(image->magick,magick,sizeof(image->magick));
+
+          /* Get HEIF image handle */
+          heif_status=heif_context_get_image_handle(heif, item_ids[item_num],&heif_image_handle);
+          if (heif_status.code == heif_error_Memory_allocation_error)
+            ThrowHEIFReaderException(ResourceLimitError, MemoryAllocationFailed, image);
+          if (heif_status.code != heif_error_Ok)
+            {
+              if (image->logging)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                      "heif_context_get_image_handle() reports error \"%s\"",
+                                      heif_status.message);
+              ThrowHEIFReaderException(CorruptImageError, AnErrorHasOccurredReadingFromFile, image);
+            }
+
+          /* Read the HEIF frame */
+          status=ReadHEIFImageFrame(heif_image_handle,decode_options,image,image_info,exception);
+
+          /* Release the HEIF image handle */
+          heif_image_handle_release(heif_image_handle);
+          heif_image_handle=(struct heif_image_handle *) NULL;
+
+          if (status == MagickFail)
+            break;
+
+          /* Quit if no more scenes requested */
+          if (image_info->subrange != 0)
+            if (image->scene >= (image_info->subimage+image_info->subrange-1))
+              break;
+        }
+      MagickFreeResourceLimitedMemory(heif_item_id *,item_ids);
+    }
+
   HEIFReadCleanup();
   CloseBlob(image);
   if (status == MagickFail)
@@ -1828,12 +2067,12 @@ static Image *ReadHEIFImage(const ImageInfo *image_info,
 ModuleExport void RegisterHEIFImage(void)
 {
 #if defined(HasHEIF)
-  static const char
-    description[] = "HEIF Image Format";
-
   static char
     version[20];
 
+  static const MagickBool
+    need_seekable_stream=MagickFalse; /* Currently use in memory blob */
+
   MagickInfo
     *entry;
 
@@ -1851,41 +2090,126 @@ ModuleExport void RegisterHEIFImage(void)
                   "heif v%u.%u.%u", heif_major,
                   heif_minor, heif_revision);
 
-  entry=SetMagickInfo("AVIF");
-  entry->decoder=(DecoderHandler) ReadHEIFImage;
-  entry->magick=(MagickHandler) IsHEIF;
-  entry->description=description;
-  entry->adjoin=False;
-  entry->seekable_stream=MagickTrue;
-  if (*version != '\0')
-    entry->version=version;
-  entry->module="HEIF";
-  entry->coder_class=PrimaryCoderClass;
-  (void) RegisterMagickInfo(entry);
+  /* AVCI (H.264) */
+  if (heif_have_decoder_for_format(heif_compression_AVC)/* || */
+      /* heif_have_encoder_for_format(heif_compression_AVC) */)
+    {
+      entry=SetMagickInfo("AVCI");
+      entry->decoder=(DecoderHandler) ReadHEIFImage;
+      entry->magick=(MagickHandler) IsHEIF;
+      entry->description="Advanced Video Coding Image Format";
+      entry->adjoin=MagickFalse;
+      entry->seekable_stream=need_seekable_stream;
+      if (*version != '\0')
+        entry->version=version;
+      entry->module="HEIF";
+      entry->coder_class=PrimaryCoderClass;
+      entry->extension_treatment=ObeyExtensionTreatment;
+      (void) RegisterMagickInfo(entry);
+    }
 
+  /* AVIF / heif_compression_AV1 */
+  if (heif_have_decoder_for_format(heif_compression_AV1)/* || */
+      /* heif_have_encoder_for_format(heif_compression_AV1) */)
+    {
+      entry=SetMagickInfo("AVIF");
+      entry->decoder=(DecoderHandler) ReadHEIFImage;
+      entry->magick=(MagickHandler) IsHEIF;
+      entry->description="AV1 Image Format";
+      entry->adjoin=MagickFalse;
+      entry->seekable_stream=need_seekable_stream;
+      if (*version != '\0')
+        entry->version=version;
+      entry->module="HEIF";
+      entry->coder_class=PrimaryCoderClass;
+      entry->extension_treatment=ObeyExtensionTreatment;
+      (void) RegisterMagickInfo(entry);
+    }
+
+  /*
+    This entry is used for anything which looks like HEIF, but is not
+    one of our directly supported subformats.
+  */
   entry=SetMagickInfo("HEIF");
   entry->decoder=(DecoderHandler) ReadHEIFImage;
   entry->magick=(MagickHandler) IsHEIF;
-  entry->description=description;
-  entry->adjoin=False;
-  entry->seekable_stream=MagickTrue;
+  entry->description="High Efficiency Image Format";
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=need_seekable_stream;
   if (*version != '\0')
     entry->version=version;
   entry->module="HEIF";
   entry->coder_class=PrimaryCoderClass;
+  entry->extension_treatment=ObeyExtensionTreatment;
   (void) RegisterMagickInfo(entry);
 
-  entry=SetMagickInfo("HEIC");
-  entry->decoder=(DecoderHandler) ReadHEIFImage;
-  entry->magick=(MagickHandler) IsHEIF;
-  entry->description=description;
-  entry->adjoin=False;
-  entry->seekable_stream=MagickTrue;
-  if (*version != '\0')
-    entry->version=version;
-  entry->module="HEIF";
-  entry->coder_class=PrimaryCoderClass;
-  (void) RegisterMagickInfo(entry);
+  /* HEIC (H.265) / heif_compression_HEVC */
+  if (heif_have_decoder_for_format(heif_compression_HEVC) /* || */
+      /* heif_have_encoder_for_format(heif_compression_HEVC) */)
+    {
+      entry=SetMagickInfo("HEIC");
+      entry->decoder=(DecoderHandler) ReadHEIFImage;
+      entry->magick=(MagickHandler) IsHEIF;
+      entry->description="High-Efficiency Image Codec in HEIF";
+      entry->adjoin=MagickFalse;
+      entry->seekable_stream=need_seekable_stream;
+      if (*version != '\0')
+        entry->version=version;
+      entry->module="HEIF";
+      entry->coder_class=PrimaryCoderClass;
+      entry->extension_treatment=ObeyExtensionTreatment;
+      (void) RegisterMagickInfo(entry);
+
+      /* Libheif supports some MP4 files */
+      entry=SetMagickInfo("MP4");
+      entry->decoder=(DecoderHandler) ReadHEIFImage;
+      entry->magick=(MagickHandler) IsHEIF;
+      entry->description="MPEG-4 Part 14 in HEIF";
+      entry->adjoin=MagickFalse;
+      entry->seekable_stream=need_seekable_stream;
+      if (*version != '\0')
+        entry->version=version;
+      entry->module="HEIF";
+      entry->coder_class=PrimaryCoderClass;
+      entry->extension_treatment=ObeyExtensionTreatment;
+      (void) RegisterMagickInfo(entry);
+    }
+
+  /* MP4 (H.264, H.265) / heif_compression_HEVC */
+  /*
+    Since HEIF image sequences are very similar to MP4 video, libheif
+    can also read and write MP4 video (without audio) with all
+    supported codecs.
+
+    heif_context_has_sequence() will return if input is a sequence.
+    Note that a file can contain still images and a sequence at the
+    same time.
+
+    heif_compression_HEVC is H.265, heif_compression_AVC is H.264, and
+    heif_compression_AV1 is AV1
+
+    video/mp4 or application/mp4
+  */
+#if 0
+  if (heif_have_decoder_for_format(heif_compression_HEVC) ||
+      heif_have_decoder_for_format(heif_compression_AVC) ||
+      heif_have_decoder_for_format(heif_compression_AV1))
+    {
+      entry=SetMagickInfo("MP4");
+      entry->decoder=(DecoderHandler) ReadHEIFImage;
+      entry->magick=(MagickHandler) IsHEIF;
+      entry->description="MP4 video container (HVEC, AVC, or AV1 compression)";
+      entry->adjoin=MagickFalse;
+      entry->seekable_stream=need_seekable_stream;
+      if (*version != '\0')
+        entry->version=version;
+      entry->module="HEIF";
+      entry->coder_class=UnstableCoderClass;
+      entry->extension_treatment=HintExtensionTreatment;
+      (void) RegisterMagickInfo(entry);
+    }
+#endif
+
 #endif /* HasHEIF */
 }
 
@@ -1911,9 +2235,11 @@ ModuleExport void RegisterHEIFImage(void)
 ModuleExport void UnregisterHEIFImage(void)
 {
 #if defined(HasHEIF)
+  (void) UnregisterMagickInfo("AVCI");
   (void) UnregisterMagickInfo("AVIF");
-  (void) UnregisterMagickInfo("HEIF");
   (void) UnregisterMagickInfo("HEIC");
+  (void) UnregisterMagickInfo("HEIF");
+  (void) UnregisterMagickInfo("MP4");
   if (heif_initialized)
     heif_deinit();
 #endif /* HasHEIF */
