@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -47,6 +47,7 @@
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /* http://msdn.microsoft.com/en-us/library/ms997538.aspx */
 
@@ -61,11 +62,11 @@ typedef struct _IconDirEntry
     height,
     colors,
     reserved;
-  
+
   magick_uint16_t
     planes,
     bits_per_pixel;
-  
+
   magick_uint32_t
     size,
     offset;
@@ -80,7 +81,7 @@ typedef struct _IconFile
     reserved,
     resource_type, /* 1 = ICON, 2 = CURSOR */
     count;
-  
+
   IconDirEntry
     directory[MaxIcons];
 } IconFile;
@@ -260,7 +261,7 @@ static Image *ReadIconImage(const ImageInfo *image_info,
       ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
     data_alloc_size=Max(data_alloc_size,icon_file.directory[i].size);
   }
-  data=MagickAllocateMemory(unsigned char *,data_alloc_size);
+  data=MagickAllocateResourceLimitedMemory(unsigned char *,data_alloc_size);
   if (data == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   for (i=0; i < icon_file.count; i++)
@@ -288,7 +289,7 @@ static Image *ReadIconImage(const ImageInfo *image_info,
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                 "Failed to seek to offset %u",
                                 icon_file.directory[i].offset);
-        MagickFreeMemory(data);
+        MagickFreeResourceLimitedMemory(unsigned char *,data);
         ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
       }
     if ((count=ReadBlob(image,icon_file.directory[i].size,data)) != icon_file.directory[i].size)
@@ -299,17 +300,17 @@ static Image *ReadIconImage(const ImageInfo *image_info,
                                   " (expected %" MAGICK_SIZE_T_F  "u bytes)",
                                   (MAGICK_SIZE_T) count,
                                   (MAGICK_SIZE_T) icon_file.directory[i].size);
-        MagickFreeMemory(data);
+        MagickFreeResourceLimitedMemory(unsigned char *,data);
         ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
       }
     format[0]='\0';
     if (memcmp(data,"\050\000\000\000",4) == 0)
-      (void) strcpy(format,"ICODIB");
+      (void) strlcpy(format,"ICODIB",sizeof(format));
     else if (memcmp(data,"\211PNG\r\n\032\n",8) == 0)
-      (void) strcpy(format,"PNG");
+      (void) strlcpy(format,"PNG",sizeof(format));
     if (format[0] == '\0')
       {
-        MagickFreeMemory(data);
+        MagickFreeResourceLimitedMemory(unsigned char *,data);
         ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
       }
     if (image->logging)
@@ -335,7 +336,7 @@ static Image *ReadIconImage(const ImageInfo *image_info,
       }
     if (icon_image == (Image *) NULL)
       {
-        MagickFreeMemory(data);
+        MagickFreeResourceLimitedMemory(unsigned char *,data);
         DestroyImageList(image);
         return((Image *) NULL);
       }
@@ -345,6 +346,7 @@ static Image *ReadIconImage(const ImageInfo *image_info,
     (void) strlcpy(icon_image->magick,image_info->magick,
                    sizeof(icon_image->magick));
     ReplaceImageInList(&image,icon_image);
+    StopTimer(&image->timer);
 
     /*
       Proceed to next image.
@@ -372,7 +374,7 @@ static Image *ReadIconImage(const ImageInfo *image_info,
   while (image->previous != (Image *) NULL)
     image=image->previous;
   CloseBlob(image);
-  MagickFreeMemory(data);
+  MagickFreeResourceLimitedMemory(unsigned char *,data);
   return(image);
 }
 
