@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2017 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -68,6 +68,7 @@
 #include "magick/pixel_cache.h"
 #include "magick/quantize.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Define declarations.
@@ -728,7 +729,7 @@ static void LogPALMHeader(const PalmHeader* palm_header)
   static const struct
   {
     unsigned short mask;
-    const char * text;
+    const char text[18];
   }
   FlagDecodes[] =
     {
@@ -849,8 +850,8 @@ GetPalmPaletteGivenBits(const unsigned int bits,
 */
 #define ThrowPALMReaderException(code_,reason_,image_) \
 do { \
-  MagickFreeMemory(one_row); \
-  MagickFreeMemory(lastrow); \
+  MagickFreeResourceLimitedMemory(unsigned char *,one_row);     \
+  MagickFreeResourceLimitedMemory(unsigned char *, lastrow);      \
   ThrowReaderException(code_,reason_,image_); \
 } while (0);
 static Image *ReadPALMImage(const ImageInfo *image_info,
@@ -902,8 +903,8 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   /*
     Open image file.
   */
-  one_row=(unsigned char*) NULL;
-  lastrow=(unsigned char*) NULL;
+  one_row=(unsigned char *) NULL;
+  lastrow=(unsigned char *) NULL;
 
   (void) memset(&palm_header,0,sizeof(palm_header));
   image=AllocateImage(image_info);
@@ -1090,6 +1091,7 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
   if (image_info->ping)
     {
       CloseBlob(image);
+      StopTimer(&image->timer);
       return(image);
     }
 
@@ -1097,14 +1099,14 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
     ThrowPALMReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
 
   alloc_size = Max(palm_header.bytes_per_row,MagickArraySize(2,image->columns));
-  one_row = MagickAllocateMemory(unsigned char *,alloc_size);
+  one_row = MagickAllocateResourceLimitedMemory(unsigned char *,alloc_size);
   if (one_row == (unsigned char *) NULL)
     ThrowPALMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   (void) memset(one_row,0,alloc_size);
   if (palm_header.compression_type == PALM_COMPRESSION_SCANLINE)
     {
       alloc_size = Max(palm_header.bytes_per_row,MagickArraySize(2,image->columns));
-      lastrow = MagickAllocateMemory(unsigned char *,alloc_size);
+      lastrow = MagickAllocateResourceLimitedMemory(unsigned char *,alloc_size);
       if (lastrow == (unsigned char *) NULL)
         ThrowPALMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
       (void) memset(lastrow,0,alloc_size);
@@ -1252,9 +1254,10 @@ static Image *ReadPALMImage(const ImageInfo *image_info,
         }
     }
 
-  MagickFreeMemory(one_row);
-  MagickFreeMemory(lastrow);
+  MagickFreeResourceLimitedMemory(unsigned char *,one_row);
+  MagickFreeResourceLimitedMemory(unsigned char *,lastrow);
   CloseBlob(image);
+  StopTimer(&image->timer);
   return(image);
 }
 
@@ -1810,14 +1813,14 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
 
   if (palm_image->compression == FaxCompression)
     {
-      lastrow = MagickAllocateMemory(unsigned char *,bytes_per_row);
+      lastrow = MagickAllocateResourceLimitedMemory(unsigned char *,bytes_per_row);
       if (lastrow == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     }
-  one_row = MagickAllocateMemory(unsigned char *,bytes_per_row);
+  one_row = MagickAllocateResourceLimitedMemory(unsigned char *,bytes_per_row);
   if (one_row == (unsigned char *) NULL)
     {
-      MagickFreeMemory(lastrow);
+      MagickFreeResourceLimitedMemory(lastrow);
       ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     }
 
@@ -1949,10 +1952,10 @@ static unsigned int WritePALMImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobMSBShort(image, count - 16);
     }
 
-  CloseBlob(image);
+  status &= CloseBlob(image);
   DestroyImage(palm_image);
-  MagickFreeMemory(one_row);
-  MagickFreeMemory(lastrow);
-  return(True);
+  MagickFreeResourceLimitedMemory(one_row);
+  MagickFreeResourceLimitedMemory(lastrow);
+  return(status);
 }
 #endif /* if ENABLE_PALM_WRITER */
