@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2021 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -52,6 +52,7 @@
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Typedef declarations.
@@ -342,11 +343,11 @@ static unsigned int IsPDB(const unsigned char *magick,const size_t length)
 %
 %
 */
-#define ThrowPDBReaderException(code_,reason_,image_) \
-{ \
-  MagickFreeResourceLimitedMemory(pixels);                   \
-  ThrowReaderException(code_,reason_,image_); \
-}
+#define ThrowPDBReaderException(code_,reason_,image_)           \
+  {                                                             \
+    MagickFreeResourceLimitedMemory(unsigned char *,pixels);    \
+    ThrowReaderException(code_,reason_,image_);                 \
+  }
 
 static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -517,7 +518,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->compression=NoCompression;
       if (ReadBlob(image,packets,(char *) pixels) != packets)
         {
-          MagickFreeResourceLimitedMemory(pixels);
+          MagickFreeResourceLimitedMemory(unsigned char *,pixels);
           ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
         }
       break;
@@ -655,7 +656,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         ThrowPDBReaderException(CorruptImageError,ImproperImageHeader,image);
       }
   }
-  MagickFreeResourceLimitedMemory(pixels);
+  MagickFreeResourceLimitedMemory(unsigned char *,pixels);
   if (EOFBlob(image))
     ThrowException(exception,CorruptImageError,UnexpectedEndOfFile,
       image->filename);
@@ -693,7 +694,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   new_comment=MagickReallocateResourceLimitedMemory(char *,comment,length+1);
                   if (new_comment == (char *) NULL)
                     {
-                      MagickFreeResourceLimitedMemory(comment);
+                      MagickFreeResourceLimitedMemory(char *,comment);
                       break;
                     }
                   comment=new_comment;
@@ -707,7 +708,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (comment == (char *) NULL)
         ThrowPDBReaderException(ResourceLimitError,MemoryAllocationFailed,image);
       (void) SetImageAttribute(image,"comment",comment);
-      MagickFreeResourceLimitedMemory(comment);
+      MagickFreeResourceLimitedMemory(char *,comment);
     }
   CloseBlob(image);
   StopTimer(&image->timer);
@@ -821,13 +822,13 @@ static unsigned char *EncodeRLE(unsigned char *destination,
   return(destination);
 }
 
-#define ThrowPDBWriterException(code_,reason_,image_) \
-{ \
-  MagickFreeResourceLimitedMemory(buffer);                     \
-  MagickFreeResourceLimitedMemory(p);                          \
-  MagickFreeResourceLimitedMemory(scanline);                   \
-  ThrowWriterException(code_,reason_,image_); \
-}
+#define ThrowPDBWriterException(code_,reason_,image_)            \
+  {                                                              \
+    MagickFreeResourceLimitedMemory(unsigned char *,buffer);     \
+    MagickFreeResourceLimitedMemory(unsigned char *,p);          \
+    MagickFreeResourceLimitedMemory(unsigned char *,scanline);   \
+    ThrowWriterException(code_,reason_,image_);                  \
+  }
 
 static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
 {
@@ -1031,8 +1032,8 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
         break;
   }
   q=EncodeRLE(q,buffer,literal,repeat);
-  MagickFreeResourceLimitedMemory(scanline);
-  MagickFreeResourceLimitedMemory(buffer);
+  MagickFreeResourceLimitedMemory(unsigned char *,scanline);
+  MagickFreeResourceLimitedMemory(unsigned char *,buffer);
   /*
     Write the Image record header.
   */
@@ -1069,9 +1070,9 @@ static unsigned int WritePDBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobMSBShort(image,pdb_image.width);
   (void) WriteBlobMSBShort(image,pdb_image.height);
   (void) WriteBlob(image,q-p,p);
-  MagickFreeResourceLimitedMemory(p);
+  MagickFreeResourceLimitedMemory(unsigned char *,p);
   if ((comment != (ImageAttribute *) NULL) && (comment->value != (char *) NULL))
     (void) WriteBlobString(image,comment->value);
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
