@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2018 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -52,6 +52,7 @@
 #include "magick/tempfile.h"
 #include "magick/utility.h"
 #include "magick/version.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -131,6 +132,7 @@ static unsigned int IsPDF(const unsigned char *magick,const size_t offset)
 %
 %
 */
+#if defined(HasGS)
 static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define MediaBox  "/MediaBox"
@@ -417,7 +419,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read Ghostscript output.
       */
-      clone_info->blob=(void *) NULL;
+      clone_info->blob=(_BlobInfoPtr_) NULL;
       clone_info->length=0;
       clone_info->magick[0]='\0';
       clone_info->subimage=0;
@@ -458,6 +460,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   return(image);
 }
+#endif /* if defined(HasGS) */
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -488,7 +491,9 @@ ModuleExport void RegisterPDFImage(void)
     *entry;
 
   entry=SetMagickInfo("EPDF");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPDFImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePDFImage;
   entry->adjoin=False;
   entry->blob_support=False;
@@ -499,7 +504,9 @@ ModuleExport void RegisterPDFImage(void)
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("PDF");
+#if defined(HasGS)
   entry->decoder=(DecoderHandler) ReadPDFImage;
+#endif /* if defined(HasGS) */
   entry->encoder=(EncoderHandler) WritePDFImage;
   entry->magick=(MagickHandler) IsPDF;
   entry->blob_support=False;
@@ -566,7 +573,7 @@ ModuleExport void UnregisterPDFImage(void)
 %
 */
 
-static char *EscapeParenthesis(const char *text)
+static char *EscapeParenthesis(const char *text, char *paren_buffer)
 {
   register char
     *p;
@@ -574,15 +581,11 @@ static char *EscapeParenthesis(const char *text)
   register long
     i;
 
-  /* FIXME: use of a static buffer here makes this function not thread safe! */
-  static char
-    buffer[MaxTextExtent];
-
   unsigned long
     escapes;
 
   escapes=0;
-  p=buffer;
+  p=paren_buffer;
   for (i=0; i < (long) Min(strlen(text),(MaxTextExtent-escapes-1)); i++)
   {
     if ((text[i] == '(') || (text[i] == ')'))
@@ -593,20 +596,20 @@ static char *EscapeParenthesis(const char *text)
     *p++=text[i];
   }
   *p='\0';
-  return(buffer);
+  return(paren_buffer);
 }
 
-#define ThrowPDFWriterException(code_,reason_,image_) \
-  {                                                   \
-    MagickFreeMemory(fax_blob);                       \
-    MagickFreeMemory(xref);                           \
-    ThrowWriterException(code_,reason_,image_);       \
+#define ThrowPDFWriterException(code_,reason_,image_)                   \
+  {                                                                     \
+    MagickFreeMemory(fax_blob);                                         \
+    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref); \
+    ThrowWriterException(code_,reason_,image_);                         \
   }
-#define ThrowPDFWriterException2(code_,reason_,image_)  \
-  {                                                     \
-    MagickFreeMemory(fax_blob);                         \
-    MagickFreeMemory(xref);                             \
-    ThrowWriterException2(code_,reason_,image_);        \
+#define ThrowPDFWriterException2(code_,reason_,image_)                  \
+  {                                                                     \
+    MagickFreeMemory(fax_blob);                                         \
+    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref); \
+    ThrowWriterException2(code_,reason_,image_);                        \
   }
 static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
 {
@@ -619,7 +622,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     date[MaxTextExtent],
     density[MaxTextExtent],
     **labels,
-    page_geometry[MaxTextExtent];
+    page_geometry[MaxTextExtent],
+    paren_buffer[MaxTextExtent];
 
   unsigned char
     *fax_blob=(unsigned char *) NULL;
@@ -663,6 +667,11 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     fax_blob_length,
     length;
 
+#if defined(HAVE_LOCALTIME_R)
+  struct tm
+    tm_buf;
+#endif /* if defined(HAVE_LOCALTIME_R) */
+
   struct tm
     *time_meridian;
 
@@ -705,7 +714,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   /*
     Allocate X ref memory.
   */
-  xref=MagickAllocateMemory(ExtendedSignedIntegralType *,
+  xref=MagickAllocateResourceLimitedMemory(ExtendedSignedIntegralType *,
                             2048*sizeof(ExtendedSignedIntegralType));
   if (xref == (ExtendedSignedIntegralType *) NULL)
     ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -721,20 +730,27 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"<<\n");
   seconds=time((time_t *) NULL);
-  time_meridian=localtime(&seconds);
+#if defined(HAVE_LOCALTIME_R)
+  time_meridian=localtime_r(&seconds,&tm_buf);
+#else
+  time_meridian=localtime(&seconds); /* Thread-unsafe version */
+#endif  /* if defined(HAVE_LOCALTIME_R) */
+
   FormatString(date,"D:%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+1900,
                time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
                time_meridian->tm_min,time_meridian->tm_sec);
   GetPathComponent(image->filename,BasePath,basename);
 
-  FormatString(buffer,"/Title (%.1024s)\n",EscapeParenthesis(basename));
+  FormatString(buffer,"/Title (%.1024s)\n",
+               EscapeParenthesis(basename,paren_buffer));
   (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/CreationDate (%.1024s)\n",date);
   (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/ModDate (%.1024s)\n",date);
   (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/Producer (%.1024s)\n",
-               EscapeParenthesis(GetMagickVersion((unsigned long *) NULL)));
+               EscapeParenthesis(GetMagickVersion((unsigned long *) NULL),
+                                 paren_buffer));
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,">>\n");
   (void) WriteBlobString(image,"endobj\n");
@@ -768,6 +784,9 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       Image
         *kid_image;
 
+      ExtendedSignedIntegralType
+        *new_xref;
+
       /*
         Predict page object id's.
       */
@@ -778,10 +797,14 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlobString(image,buffer);
           kid_image=kid_image->next;
         }
-      MagickReallocMemory(ExtendedSignedIntegralType *,xref,
-                          (count+2048)*sizeof(ExtendedSignedIntegralType));
-      if (xref == (ExtendedSignedIntegralType *) NULL)
-        ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+      new_xref=MagickReallocateResourceLimitedArray(ExtendedSignedIntegralType *,xref,
+                                                    (size_t) count+2048,sizeof(ExtendedSignedIntegralType));
+      if (new_xref == (ExtendedSignedIntegralType *) NULL)
+        {
+          MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref);
+          ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+        }
+      xref=new_xref;
     }
   (void) WriteBlobString(image,"]\n");
   FormatString(buffer,"/Count %lu\n",(count-pages_id)/ObjectsPerImage);
@@ -983,7 +1006,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       FormatString(buffer,"/ProcSet %lu 0 R >>\n",object+3);
       (void) WriteBlobString(image,buffer);
       FormatString(buffer,"/MediaBox [0 0 %ld %ld]\n",
-                   media_info.width,media_info.height);
+                   (long int)(((media_info.width * dx_resolution) / x_resolution) + 0.5),
+                   (long int)(((media_info.height * dy_resolution) / y_resolution) + 0.5));
       (void) WriteBlobString(image,buffer);
       FormatString(buffer,"/CropBox [%ld %ld %ld %ld]\n",geometry.x,geometry.y,
                    geometry.x+geometry.width,geometry.y+geometry.height);
@@ -1017,8 +1041,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
               FormatString(buffer,"/F%lu %g Tf\n",image->scene,
                            image_info->pointsize);
               (void) WriteBlobString(image,buffer);
-              FormatString(buffer,"%ld %g Td\n",geometry.x,geometry.y+
-                           geometry.height+i*image_info->pointsize+12);
+              FormatString(buffer,"%ld %g Td\n",geometry.x,(double) geometry.y+
+                           (double) geometry.height+i*(double) image_info->pointsize+12);
               (void) WriteBlobString(image,buffer);
               FormatString(buffer,"(%.1024s) Tj\n",labels[i]);
               (void) WriteBlobString(image,buffer);
@@ -1208,7 +1232,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                   Allocate pixel array.
                 */
                 length=number_pixels;
-                pixels=MagickAllocateMemory(unsigned char *,length);
+                pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
                 if (pixels == (unsigned char *) NULL)
                   ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,
                                           image);
@@ -1248,11 +1272,11 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                     status=LZWEncodeImage(image,length,pixels);
                   else
                     status=PackbitsEncodeImage(image,length,pixels);
-                MagickFreeMemory(pixels);
+                MagickFreeResourceLimitedMemory(unsigned char *,pixels);
                 if (!status)
                   {
                     CloseBlob(image);
-                    MagickFreeMemory(xref);
+                    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref);
                     return(MagickFail);
                   }
                 break;
@@ -1318,8 +1342,8 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                 /*
                   Allocate pixel array.
                 */
-                length=(image->colorspace == CMYKColorspace ? 4 : 3)*number_pixels;
-                pixels=MagickAllocateMemory(unsigned char *,length);
+                length=(size_t) (image->colorspace == CMYKColorspace ? 4 : 3)*number_pixels;
+                pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
                 if (pixels == (unsigned char *) NULL)
                   ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,
                                           image);
@@ -1371,10 +1395,11 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                     status=LZWEncodeImage(image,length,pixels);
                   else
                     status=PackbitsEncodeImage(image,length,pixels);
-                MagickFreeMemory(pixels);
+                MagickFreeResourceLimitedMemory(unsigned char *,pixels);
                 if (!status)
                   {
                     CloseBlob(image);
+                    MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref);
                     return(False);
                   }
                 break;
@@ -1437,7 +1462,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                     Allocate pixel array.
                   */
                   length=number_pixels;
-                  pixels=MagickAllocateMemory(unsigned char *,length);
+                  pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
                   if (pixels == (unsigned char *) NULL)
                     ThrowPDFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
                   /*
@@ -1474,10 +1499,11 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
                       status=LZWEncodeImage(image,length,pixels);
                     else
                       status=PackbitsEncodeImage(image,length,pixels);
-                  MagickFreeMemory(pixels);
+                  MagickFreeResourceLimitedMemory(unsigned char *,pixels);
                   if (!status)
                     {
                       CloseBlob(image);
+                      MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref);
                       return(False);
                     }
                   break;
@@ -1645,10 +1671,10 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   FormatString(buffer,"%lu\n",(unsigned long) offset);
   (void) WriteBlobString(image,buffer);
   (void) WriteBlobString(image,"%%EOF\n");
-  MagickFreeMemory(xref);
-  CloseBlob(image);
+  MagickFreeResourceLimitedMemory(ExtendedSignedIntegralType *,xref);
+  status &= CloseBlob(image);
   MagickFreeMemory(fax_blob);
-  return(MagickPass);
+  return(status);
 }
 
 #if defined(HasZLIB)
@@ -1722,7 +1748,7 @@ static unsigned int ZLIBEncodeImage(Image *image,const size_t length,
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   compressed_packets=(unsigned long) (1.001*length+12);
-  compressed_pixels=MagickAllocateMemory(unsigned char *,compressed_packets);
+  compressed_pixels=MagickAllocateResourceLimitedMemory(unsigned char *,compressed_packets);
   if (compressed_pixels == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
       (char *) NULL);
@@ -1745,11 +1771,14 @@ static unsigned int ZLIBEncodeImage(Image *image,const size_t length,
       compressed_packets=stream.total_out;
     }
   if (status)
-    ThrowBinaryException(CoderError,UnableToZipCompressImage,(char *) NULL)
+    {
+      (void) deflateEnd(&stream);
+      ThrowBinaryException(CoderError,UnableToZipCompressImage,(char *) NULL);
+    }
   else
     for (i=0; i < (long) compressed_packets; i++)
       (void) WriteBlobByte(image,compressed_pixels[i]);
-  MagickFreeMemory(compressed_pixels);
+  MagickFreeResourceLimitedMemory(unsigned char *,compressed_pixels);
   return(!status);
 }
 #endif
