@@ -48,6 +48,7 @@
 #include "magick/tempfile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   GraphicsMagick Macintosh PICT Methods.
@@ -958,7 +959,7 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,
           p=ExpandBuffer(expand_buffer,scanline,&number_pixels,bits_per_pixel);
           (void) memcpy(q,p,number_pixels);
         }
-      MagickFreeResourceLimitedMemory(scanline);
+      MagickFreeResourceLimitedMemory(unsigned char *,scanline);
       return(pixels);
     }
   /*
@@ -1078,13 +1079,13 @@ static unsigned char *DecodeImage(const ImageInfo *image_info,
             j+=bytes_per_pixel+1;
           }
     }
-  MagickFreeResourceLimitedMemory(scanline);
+  MagickFreeResourceLimitedMemory(unsigned char *,scanline);
   return (pixels);
 
  decode_error_exit:
 
-  MagickFreeResourceLimitedMemory(scanline);
-  MagickFreeResourceLimitedMemory(pixels);
+  MagickFreeResourceLimitedMemory(unsigned char *,scanline);
+  MagickFreeResourceLimitedMemory(unsigned char *,pixels);
   return (unsigned char *) NULL;
 }
 
@@ -1246,12 +1247,12 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
   length=(q-pixels);
   if (bytes_per_line > 250)
     {
-      (void) WriteBlobMSBShort(image,(const magick_uint16_t) length);
+      (void) WriteBlobMSBShort(image,(magick_uint16_t) length);
       length+=2;
     }
   else
     {
-      (void) WriteBlobByte(image,(const magick_uint8_t) length);
+      (void) WriteBlobByte(image,(magick_uint8_t) length);
       length++;
     }
   while (q != pixels)
@@ -1817,7 +1818,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                                               image->columns,image->rows))
                     break;
             }
-            MagickFreeResourceLimitedMemory(pixels);
+            MagickFreeResourceLimitedMemory(unsigned char *,pixels);
             if (tile_image->exception.severity > image->exception.severity)
               CopyException(&image->exception,&tile_image->exception);
             if ((tile_image->exception.severity < ErrorException) && (jpeg == False))
@@ -1872,7 +1873,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   break;
                 if (SetImageProfile(image,"ICM",info,length) == MagickFail)
                   ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                MagickFreeResourceLimitedMemory(info);
+                MagickFreeResourceLimitedMemory(unsigned char *,info);
                 break;
               }
               case 0x1f2:
@@ -1881,13 +1882,13 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   break;
                 if (SetImageProfile(image,"IPTC",info,length) == MagickFail)
                   ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-                MagickFreeResourceLimitedMemory(info);
+                MagickFreeResourceLimitedMemory(unsigned char *,info);
                 break;
               }
               default:
                 break;
             }
-            MagickFreeResourceLimitedMemory(info);
+            MagickFreeResourceLimitedMemory(unsigned char *,info);
             break;
           }
           default:
@@ -1946,12 +1947,12 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
             blob=blob_alloc;
             clone_info=CloneImageInfo(image_info);
-            clone_info->blob=(void *) NULL;
+            clone_info->blob=(_BlobInfoPtr_) NULL;
             clone_info->length=0;
             (void) strlcpy(clone_info->filename,"JPEG:",sizeof(clone_info->filename));
             if (ReadBlobZC(image,blob_alloc_size,&blob) != blob_alloc_size)
               {
-                MagickFreeResourceLimitedMemory(blob_alloc);
+                MagickFreeResourceLimitedMemory(void *,blob_alloc);
                 ThrowPICTReaderException(CorruptImageError,UnexpectedEndOfFile,image);
               }
             if (blob != blob_alloc)
@@ -1963,7 +1964,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             tile_image=BlobToImage(clone_info, blob, blob_alloc_size, &image->exception );
             DestroyImageInfo(clone_info);
             clone_info=(ImageInfo *) NULL;
-            MagickFreeResourceLimitedMemory(blob_alloc);
+            MagickFreeResourceLimitedMemory(void *,blob_alloc);
           }
         if (tile_image == (Image *) NULL)
           continue;
@@ -2147,11 +2148,11 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %
 */
-#define LiberatePICTAllocations()               \
-  {                                             \
-    MagickFreeResourceLimitedMemory(buffer);                   \
-    MagickFreeResourceLimitedMemory(packed_scanline);          \
-    MagickFreeResourceLimitedMemory(scanline);                 \
+#define LiberatePICTAllocations()                                       \
+  {                                                                     \
+    MagickFreeResourceLimitedMemory(unsigned char *,buffer);            \
+    MagickFreeResourceLimitedMemory(unsigned char *,packed_scanline);   \
+    MagickFreeResourceLimitedMemory(unsigned char *,scanline);          \
   }
 #define ThrowPICTWriterException(code_,reason_,image_)  \
   {                                                     \
@@ -2360,7 +2361,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0x1f2);
-      (void) WriteBlobMSBShort(image,(const magick_uint16_t) profile_length+4);
+      (void) WriteBlobMSBShort(image,(magick_uint16_t) profile_length+4);
       (void) WriteBlobString(image,"8BIM");
       (void) WriteBlob(image,profile_length,
                        profile_info);
@@ -2373,7 +2374,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     {
       (void) WriteBlobMSBShort(image,0xa1);
       (void) WriteBlobMSBShort(image,0xe0);
-      (void) WriteBlobMSBShort(image,(const magick_uint16_t) profile_length+4);
+      (void) WriteBlobMSBShort(image,(magick_uint16_t) profile_length+4);
       (void) WriteBlobMSBLong(image,0x00000000UL);
       (void) WriteBlob(image,profile_length,
                        profile_info);
@@ -2422,7 +2423,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
           return(False);
         }
       (void) WriteBlobMSBShort(image,PictJPEGOp);
-      (void) WriteBlobMSBLong(image,(const magick_uint16_t) length+154);
+      (void) WriteBlobMSBLong(image,(magick_uint16_t) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBLong(image,0x00010000UL);
       (void) WriteBlobMSBLong(image,0x00000000UL);
@@ -2616,9 +2617,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   offset=TellBlob(image);
   (void) SeekBlob(image,512,SEEK_SET);
   (void) WriteBlobMSBShort(image,(unsigned long) offset);
-  MagickFreeResourceLimitedMemory(scanline);
-  MagickFreeResourceLimitedMemory(packed_scanline);
-  MagickFreeResourceLimitedMemory(buffer);
+  MagickFreeResourceLimitedMemory(unsigned char *,scanline);
+  MagickFreeResourceLimitedMemory(unsigned char *,packed_scanline);
+  MagickFreeResourceLimitedMemory(unsigned char *,buffer);
   status &= CloseBlob(image);
   return(status);
 }
