@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2019 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -48,6 +48,7 @@
 #include "magick/monitor.h"
 #include "magick/tempfile.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 #if defined(HasTIFF)
 #define CCITTParam  "-1"
 #else
@@ -62,9 +63,17 @@ static unsigned int
 
 #if defined(HasTIFF)
 #if defined(HAVE_TIFFCONF_H)
-#include "tiffconf.h"
+#  include "tiffconf.h"
 #endif
 #include "tiffio.h"
+
+#if (TIFFLIB_VERSION >= 20201219)
+#  undef uint16
+#  define uint16 uint16_t
+#  undef uint32
+#  define uint32 uint32_t
+#endif /* TIFFLIB_VERSION */
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -172,7 +181,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
   for (i=1; i < (long) TIFFNumberOfStrips(tiff); i++)
     if (byte_count[i] > strip_size)
       strip_size=byte_count[i];
-  buffer=MagickAllocateMemory(unsigned char *,strip_size);
+  buffer=MagickAllocateResourceLimitedMemory(unsigned char *,strip_size);
   if (buffer == (unsigned char *) NULL)
     {
       TIFFClose(tiff);
@@ -194,7 +203,7 @@ static unsigned int Huffman2DEncodeImage(const ImageInfo *image_info,
       Ascii85Encode(image,(unsigned long) buffer[j]);
     Ascii85Flush(image);
   }
-  MagickFreeMemory(buffer);
+  MagickFreeResourceLimitedMemory(unsigned char *,buffer);
   TIFFClose(tiff);
   (void) LiberateTemporaryFile(filename);
   return(True);
@@ -511,26 +520,32 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
         /*
           Output Postscript header.
         */
-        if (LocaleCompare(image_info->magick,"PS2") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+ #if defined(HAVE_CTIME_R)
+        char time_buf[26];
+#endif /* defined(HAVE_CTIME_R) */
+       if (LocaleCompare(image_info->magick,"PS2") == 0)
+         (void) strlcpy(buffer,"%!PS-Adobe-3.0\n",sizeof(buffer));
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) strlcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",sizeof(buffer));
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (GraphicsMagick)\n");
         FormatString(buffer,"%%%%Title: (%.1024s)\n",image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) strlcpy(date,ctime(&timer),MaxTextExtent);
+#if defined(HAVE_CTIME_R)
+        (void) strlcpy(date,ctime_r(&timer,time_buf),MaxTextExtent);
+#else
+        (void) strlcpy(date,ctime(&timer),MaxTextExtent); /* Thread-unsafe version */
+#endif /* defined(HAVE_CTIME_R) */
         date[strlen(date)-1]='\0';
         FormatString(buffer,"%%%%CreationDate: (%.1024s)\n",date);
         (void) WriteBlobString(image,buffer);
         bounds.x1=geometry.x;
         bounds.y1=geometry.y;
-        bounds.x2=geometry.x+geometry.width;
-        bounds.y2=geometry.y+geometry.height+text_size;
+        bounds.x2=geometry.x+(size_t) geometry.width;
+        bounds.y2=geometry.y+(size_t) geometry.height+text_size;
         if (image_info->adjoin && (image->next != (Image *) NULL))
-          (void) strcpy(buffer,"%%BoundingBox: (atend)\n");
+          (void) strlcpy(buffer,"%%BoundingBox: (atend)\n",sizeof(buffer));
         else
           FormatString(buffer,"%%%%BoundingBox: %g %g %g %g\n",
             floor(bounds.x1+0.5),floor(bounds.y1+0.5),ceil(bounds.x2-0.5),
@@ -548,7 +563,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (!image_info->adjoin)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) strlcpy(buffer,"%%Pages: 1\n",sizeof(buffer));
             else
               FormatString(buffer,"%%%%Pages: %lu\n",(unsigned long)
                 image_list_length);
@@ -651,7 +666,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                        "      {\n"
                        "        /DataSource pixel_stream /%.1024s filter\n"
                        "        <<\n"
-                       "           /K "CCITTParam"\n"
+                       "           /K " CCITTParam "\n"
                        "           /Columns columns\n"
                        "           /Rows rows\n"
                        "        >> /CCITTFaxDecode filter\n"
@@ -757,10 +772,10 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       bounds.x1=geometry.x;
     if (geometry.y < bounds.y1)
       bounds.y1=geometry.y;
-    if ((geometry.x+(long) geometry.width-1) > bounds.x2)
-      bounds.x2=geometry.x+geometry.width-1;
-    if ((geometry.y+(long) (geometry.height+text_size)-1) > bounds.y2)
-      bounds.y2=geometry.y+(geometry.height+text_size)-1;
+    if ((geometry.x+(size_t) geometry.width-1) > bounds.x2)
+      bounds.x2=geometry.x+(size_t) geometry.width-1;
+    if ((geometry.y+((size_t) geometry.height+text_size)-1) > bounds.y2)
+      bounds.y2=geometry.y+((size_t) geometry.height+text_size)-1;
     attribute=GetImageAttribute(image,"label");
     if (attribute != (const ImageAttribute *) NULL)
       (void) WriteBlobString(image,"%%PageResources: font Times-Roman\n");
@@ -850,7 +865,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               Allocate pixel array.
             */
             length=number_pixels;
-            pixels=MagickAllocateMemory(unsigned char *,length);
+            pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
             if (pixels == (unsigned char *) NULL)
               ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
                 image);
@@ -885,7 +900,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               status=LZWEncodeImage(image,length,pixels);
             else
               status=PackbitsEncodeImage(image,length,pixels);
-            MagickFreeMemory(pixels);
+            MagickFreeResourceLimitedMemory(unsigned char *,pixels);
             if (!status)
               {
                 CloseBlob(image);
@@ -960,9 +975,9 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
               /*
                 Allocate pixel array.
               */
-              length=(image->colorspace == CMYKColorspace ? 4 : 3)*
-                number_pixels;
-              pixels=MagickAllocateMemory(unsigned char *,length);
+              length=MagickArraySize(image->colorspace == CMYKColorspace ? 4 : 3,
+                number_pixels);
+              pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
               /*
@@ -1020,7 +1035,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                   CloseBlob(image);
                   return(False);
                 }
-              MagickFreeMemory(pixels);
+              MagickFreeResourceLimitedMemory(unsigned char *,pixels);
               break;
             }
             case NoCompression:
@@ -1108,7 +1123,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 Allocate pixel array.
               */
               length=number_pixels;
-              pixels=MagickAllocateMemory(unsigned char *,length);
+              pixels=MagickAllocateResourceLimitedMemory(unsigned char *,length);
               if (pixels == (unsigned char *) NULL)
                 ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
               /*
@@ -1140,7 +1155,7 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
                 status=LZWEncodeImage(image,length,pixels);
               else
                 status=PackbitsEncodeImage(image,length,pixels);
-              MagickFreeMemory(pixels);
+              MagickFreeResourceLimitedMemory(unsigned char *,pixels);
               if (!status)
                 {
                   CloseBlob(image);
@@ -1219,6 +1234,6 @@ static unsigned int WritePS2Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,buffer);
     }
   (void) WriteBlobString(image,"%%EOF\n");
-  CloseBlob(image);
-  return(True);
+  status &= CloseBlob(image);
+  return(status);
 }
