@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2025 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -43,6 +43,7 @@
 #include "magick/magick.h"
 #include "magick/monitor.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -81,6 +82,12 @@
 { \
     field[sizeof(field)-1]='\0'; \
 }
+
+#define ThrowRLAReaderException(code_,reason_,image_) \
+do { \
+  MagickFreeResourceLimitedMemory(magick_uint32_t *,scanlines); \
+  ThrowReaderException(code_,reason_,image_); \
+} while (0);
 static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   typedef struct _WindowFrame
@@ -185,13 +192,13 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     number_channels,
     runlength;
 
-  long
+  unsigned long
     y;
 
   magick_uint32_t
-    *scanlines;
+    *scanlines=0;
 
-  register long
+  register unsigned long
     i,
     x;
 
@@ -216,6 +223,10 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickPassFail
     status;
 
+  magick_off_t
+    current_offset,
+    file_size;
+
   /*
     Open image file.
   */
@@ -227,6 +238,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+  file_size=GetBlobSize(image);
   is_rla3=MagickFalse;
   memset(&rla_info,0,sizeof(rla_info));
   memset(&rla3_extra_info,0,sizeof(rla3_extra_info));
@@ -311,14 +323,14 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       NULLTerminateASCIIField(rlb_extra_info.space);
     }
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+    ThrowRLAReaderException(CorruptImageError,UnexpectedEndOfFile,image);
 
   /*
     Verify revision.
   */
 
 /*   if (rla3_extra_info.revision != 0xFFFE) */
-/*     ThrowReaderException(CorruptImageError,ImproperImageHeader,image); */
+/*     ThrowRLAReaderException(CorruptImageError,ImproperImageHeader,image); */
 
   /*
     Verify dimensions.
@@ -332,13 +344,13 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                           (int) rla_info.active_window.bottom);
   if ((((long) rla_info.active_window.right - rla_info.active_window.left) < 0) ||
       (((long) rla_info.active_window.top-rla_info.active_window.bottom) < 0))
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+    ThrowRLAReaderException(CorruptImageError,ImproperImageHeader,image);
 
   if (image->logging)
     {
       const char
         *storage_type = "Unknown";
-      
+
       switch (rla_info.storage_type)
         {
         case 0:
@@ -410,7 +422,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
                                 "Time           : %.12s", rla3_extra_info.time);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                 "Filter         : %.32s", rla3_extra_info.filter);
-          
+
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                 "BitsPerChannel : %u", rla3_extra_info.bits_per_channel);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -446,14 +458,31 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
 
   if ((rla_info.storage_type != 0) || (rla_info.storage_type > 3))
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+    ThrowRLAReaderException(CorruptImageError,ImproperImageHeader,image);
 
   if (rla_info.storage_type != 0)
-    ThrowReaderException(CoderError,DataStorageTypeIsNotSupported,image);
+    ThrowRLAReaderException(CoderError,DataStorageTypeIsNotSupported,image);
 
   if (LocaleNCompare(rla_info.chan,"rgb",3) != 0)
-    ThrowReaderException(CoderError,ColorTypeNotSupported,image);
-                          
+    ThrowRLAReaderException(CoderError,ColorTypeNotSupported,image);
+
+  if (rla_info.number_channels > 3)
+    {
+      if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Unsupported number of color channels: %u",
+                                  rla_info.number_channels);
+      ThrowRLAReaderException(CorruptImageError,UnsupportedNumberOfPlanes,image);
+    }
+  if (rla_info.number_matte_channels > 1)
+    {
+      if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Unsupported number of matte channels: %u",
+                                  rla_info.number_matte_channels);
+      ThrowRLAReaderException(CorruptImageError,UnsupportedNumberOfPlanes,image);
+    }
+
   /*
     Initialize image structure.
   */
@@ -472,18 +501,19 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
 
   if (CheckImagePixelLimits(image, exception) != MagickPass)
-    ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+    ThrowRLAReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
 
   number_channels=rla_info.number_channels+rla_info.number_matte_channels;
-  scanlines=MagickAllocateArray(magick_uint32_t *,image->rows,sizeof(magick_uint32_t));
+  scanlines=MagickAllocateResourceLimitedArray(magick_uint32_t *,image->rows,sizeof(magick_uint32_t));
   if (scanlines == (magick_uint32_t *) NULL)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowRLAReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   if (*rla_info.description != '\0')
     (void) SetImageAttribute(image,"comment",rla_info.description);
   /*
     Read offsets to each scanline data.
   */
-  for (i=0; i < (long) image->rows; i++)
+  current_offset=TellBlob(image);
+  for (i=0; i < image->rows; i++)
     {
       scanlines[i]=(magick_uint32_t) ReadBlobMSBLong(image);
 #if 0
@@ -491,14 +521,37 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                               "scanline[%ld] = %lu",i,(unsigned long) scanlines[i]);
 #endif
+      if ((magick_off_t) scanlines[i] > file_size)
+        {
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "scanline[%ld] offset %lu is beyond end of file",
+                                  i,(unsigned long) scanlines[i]);
+          ThrowRLAReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+        }
+      if ((magick_off_t) scanlines[i] < current_offset)
+        {
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "scanline[%ld] offset %lu is too small!",
+                                  i,(unsigned long) scanlines[i]);
+          ThrowRLAReaderException(CorruptImageError,ImproperImageHeader,image);
+        }
+      if ((i != 0) && (scanlines[i-1] == scanlines[i]))
+        {
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "scanline[%ld] offset %lu is a duplicate!",
+                                  i,(unsigned long) scanlines[i]);
+        }
     }
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+    ThrowRLAReaderException(CorruptImageError,UnexpectedEndOfFile,image);
   /*
     Read image data.
   */
   x=0;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < image->rows; y++)
   {
     if (SeekBlob(image,scanlines[image->rows-y-1],SEEK_SET) == -1)
       {
@@ -529,6 +582,8 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             while (runlength < 0)
             {
+              if (x > image->rows*image->columns*number_channels)
+                  ThrowRLAReaderException(CorruptImageError,UnableToRunlengthDecodeImage,image);
               q=GetImagePixels(image,(long) (x % image->columns),
                                (long) (y % image->columns),1,1);
               if (q == (PixelPacket *) NULL)
@@ -589,6 +644,8 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         runlength++;
         do
         {
+          if (x > image->rows*image->columns*number_channels)
+            ThrowRLAReaderException(CorruptImageError,UnableToRunlengthDecodeImage,image);
           q=GetImagePixels(image,(long) (x % image->columns),
             (long) (y % image->columns),1,1);
           if (q == (PixelPacket *) NULL)
@@ -643,7 +700,7 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (QuantumTick(y,image->rows))
       if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
                                   image->filename,
-				  image->columns,image->rows))
+                                  image->columns,image->rows))
         {
           status=MagickFail;
           break;
@@ -652,8 +709,10 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
   }
   if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+    ThrowRLAReaderException(CorruptImageError,UnexpectedEndOfFile,image);
   CloseBlob(image);
+  MagickFreeResourceLimitedMemory(magick_uint32_t *,scanlines);
+  StopTimer(&image->timer);
   return(image);
 }
 
