@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2013-2023 GraphicsMagick Group
+% Copyright (C) 2013-2025 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -48,6 +48,7 @@
 #include "magick/profile.h"
 #include "magick/tsd.h"
 #include "magick/utility.h"
+#include "magick/static.h"
 
 /*
   Forward declarations.
@@ -102,6 +103,7 @@ static unsigned int WriteWEBPImage(const ImageInfo *,Image *);
 #endif
 #if WEBP_ENCODER_ABI_VERSION >= 0x0209 /* >= 0.5.0 */
 #  define SUPPORT_WEBP_MUX
+#  define SUPPORT_WEBP_EXACT
 #include <webp/mux.h>
 #endif
 #if WEBP_ENCODER_ABI_VERSION >= 0x020e /* >= 0.6.0 */
@@ -193,12 +195,12 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
   count=(long) ReadBlob(image,length,(char *) stream);
   if (count != (size_t) length)
     {
-      MagickFreeResourceLimitedMemory(stream);
+      MagickFreeResourceLimitedMemory(unsigned char *,stream);
       ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,image);
     }
   if ((webp_status=WebPGetFeatures(stream,length,&stream_features)) != VP8_STATUS_OK)
     {
-      MagickFreeResourceLimitedMemory(stream);
+      MagickFreeResourceLimitedMemory(unsigned char *,stream);
 
       switch (webp_status)
         {
@@ -251,14 +253,14 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
   image->matte=(stream_features.has_alpha ? MagickTrue : MagickFalse);
   if (image->ping)
     {
-      MagickFreeResourceLimitedMemory(stream);
+      MagickFreeResourceLimitedMemory(unsigned char *,stream);
       CloseBlob(image);
       StopTimer(&image->timer);
       return(image);
     }
   if (CheckImagePixelLimits(image, exception) != MagickPass)
     {
-      MagickFreeResourceLimitedMemory(stream);
+      MagickFreeResourceLimitedMemory(unsigned char *,stream);
       ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
     }
   if (image->matte)
@@ -271,7 +273,7 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
                                            &stream_features.height);
   if (pixels == (unsigned char *) NULL)
     {
-      MagickFreeResourceLimitedMemory(stream);
+      MagickFreeResourceLimitedMemory(unsigned char *,stream);
       ThrowReaderException(CoderError,NoDataReturned,image);
     }
 
@@ -343,7 +345,7 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
                             MAGICK_JPEG_APP1_EXIF_HEADER_SIZE);
             (void) memcpy((void *) (profile+header_size),flag_data.bytes,flag_data.size);
             SetImageProfile(image,"EXIF",profile,profile_size);
-            MagickFreeResourceLimitedMemory(profile);
+            MagickFreeResourceLimitedMemory(unsigned char *,profile);
           }
       }
 
@@ -365,7 +367,7 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
   */
   free(pixels);
   pixels=(unsigned char *) NULL;
-  MagickFreeResourceLimitedMemory(stream);
+  MagickFreeResourceLimitedMemory(unsigned char *,stream);
   CloseBlob(image);
 
   /*
@@ -676,7 +678,13 @@ static unsigned int WriteWEBPImage(const ImageInfo *image_info,Image *image)
     configure.quality = (float) image_info->quality;
 
   if ((value=AccessDefinition(image_info,"webp","lossless")))
-    configure.lossless=(LocaleCompare(value,"TRUE") == 0 ? 1 : 0);
+    {
+      configure.lossless=(LocaleCompare(value,"TRUE") == 0 ? 1 : 0);
+#if defined(SUPPORT_WEBP_EXACT)
+      /* Preserve RGB channels in 100% transparent areas */
+      configure.exact=1;
+#endif
+    }
   if ((value=AccessDefinition(image_info,"webp","method")))
     configure.method=MagickAtoI(value);
   if ((value=AccessDefinition(image_info,"webp","image-hint")))
@@ -739,6 +747,13 @@ static unsigned int WriteWEBPImage(const ImageInfo *image_info,Image *image)
 #if defined(SUPPORT_CONFIG_USE_SHARP_YUV)
   if ((value=AccessDefinition(image_info,"webp","use-sharp-yuv")))
     configure.use_sharp_yuv=(LocaleCompare(value,"TRUE") == 0 ? 1 : 0);
+#endif
+#if defined(SUPPORT_WEBP_EXACT)
+  if ((value=AccessDefinition(image_info,"webp","exact")))
+    {
+      /* Preserve RGB channels in 100% transparent areas */
+      configure.exact=(LocaleCompare(value,"TRUE") == 0 ? 1 : 0);
+    }
 #endif
   if (WebPValidateConfig(&configure) != 1)
     ThrowWriterException(CoderError,WebPInvalidConfiguration,image);
@@ -767,7 +782,6 @@ static unsigned int WriteWEBPImage(const ImageInfo *image_info,Image *image)
               p++;
             }
         }
-
     }
   else
     {
@@ -806,7 +820,7 @@ static unsigned int WriteWEBPImage(const ImageInfo *image_info,Image *image)
         webp_status=WebPPictureImportRGB(&picture,pixels,3*picture.width);
       else
         webp_status=WebPPictureImportRGBA(&picture,pixels,4*picture.width);
-      MagickFreeResourceLimitedMemory(pixels);
+      MagickFreeResourceLimitedMemory(unsigned char *,pixels);
     }
 
   if (webp_status)
