@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2019 GraphicsMagick Group
+% Copyright (C) 2003-2022 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -647,7 +647,7 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
   */
   window_attributes.width=XDisplayWidth(display,XDefaultScreen(display));
   window_attributes.height=XDisplayHeight(display,XDefaultScreen(display));
-  (void) strcpy(visual_type,"default");
+  (void) strlcpy(visual_type,"default",sizeof(visual_type));
   status=XGetWindowAttributes(display,window_info.id,&window_attributes);
   if (status != False)
     FormatString(visual_type,"0x%lx",
@@ -719,8 +719,8 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
         }
     }
   if (resources.map_type == (char *) NULL)
-    if ((visual_info->class != TrueColor) &&
-        (visual_info->class != DirectColor))
+    if ((visual_info->klass != TrueColor) &&
+        (visual_info->klass != DirectColor))
       {
         Image
           *next;
@@ -767,16 +767,16 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
   for (scene=0; scene < (int) number_scenes; scene++)
   {
     if ((resource_info->map_type != (char *) NULL) ||
-        (visual_info->class == TrueColor) ||
-        (visual_info->class == DirectColor))
+        (visual_info->klass == TrueColor) ||
+        (visual_info->klass == DirectColor))
       (void) SetImageType(image_list[scene],TrueColorType);
     if ((display_image->columns < image_list[scene]->columns) &&
         (display_image->rows < image_list[scene]->rows))
       display_image=image_list[scene];
   }
   if ((resource_info->map_type != (char *) NULL) ||
-      (visual_info->class == TrueColor) ||
-      (visual_info->class == DirectColor))
+      (visual_info->klass == TrueColor) ||
+      (visual_info->klass == DirectColor))
     (void) SetImageType(display_image,TrueColorType);
   MagickXMakeStandardColormap(display,visual_info,&resources,display_image,map_info,
     &pixel);
@@ -825,7 +825,7 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
     MagickFatalError(XServerFatalError,UnableToCreateXImage,(char *) NULL);
   window_info.x=0;
   window_info.y=0;
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     {
       (void) LogMagickEvent(X11Event,GetMagickModule(),
         "Image: %.1024s[%lu] %lux%lu ",image_list[0]->filename,
@@ -925,8 +925,8 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
     window_info.pixmap=(Pixmap) NULL;
     window_info.matte_pixmap=(Pixmap) NULL;
     if ((resources.map_type != (char *) NULL) ||
-        (visual_info->class == TrueColor) ||
-        (visual_info->class == DirectColor))
+        (visual_info->klass == TrueColor) ||
+        (visual_info->klass == DirectColor))
       if (image_list[scene]->storage_class == PseudoClass)
         {
           /*
@@ -941,7 +941,7 @@ MagickExport void MagickXAnimateBackgroundImage(Display *display,
       (unsigned int) image_list[scene]->rows);
     if (status == False)
       MagickFatalError(XServerFatalError,UnableToCreateXImage,(char *) NULL);
-    if (IsEventLogging())
+    if (IsEventLogged(X11Event))
       {
         (void) LogMagickEvent(X11Event,GetMagickModule(),
           "Image: [%lu] %.1024s %lux%lu ",image_list[scene]->scene,
@@ -1264,7 +1264,11 @@ MagickXAnimateImages(Display *display,
           (chdir(working_directory) != 0))
         MagickFatalError(ConfigureFatalError,UnableToRestoreCurrentDirectory,
                          NULL);
-      monitor_handler=SetMonitorHandler(MagickXMagickMonitor);
+#if defined(MAGICK_USE_XMAGICK_MONITOR) && MAGICK_USE_XMAGICK_MONITOR
+      if ((resource_info->image_info->progress) &&
+          (monitor_handler == (MonitorHandler) NULL))
+        monitor_handler=SetMonitorHandler(MagickXMagickMonitor);
+#endif /* if defined(MAGICK_USE_XMAGICK_MONITOR) && MAGICK_USE_XMAGICK_MONITOR */
       warning_handler=resource_info->display_warnings ?
         SetErrorHandler(MagickXWarning) : SetErrorHandler((ErrorHandler) NULL);
       warning_handler=resource_info->display_warnings ?
@@ -1361,8 +1365,8 @@ MagickXAnimateImages(Display *display,
         }
     }
   if (resource_info->map_type == (char *) NULL)
-    if ((visual_info->class != TrueColor) &&
-        (visual_info->class != DirectColor))
+    if ((visual_info->klass != TrueColor) &&
+        (visual_info->klass != DirectColor))
       {
         Image
           *next;
@@ -1410,14 +1414,14 @@ MagickXAnimateImages(Display *display,
   for (scene=0; scene < (long) number_scenes; scene++)
   {
     if ((resource_info->map_type != (char *) NULL) ||
-        (visual_info->class == TrueColor) ||
-        (visual_info->class == DirectColor))
+        (visual_info->klass == TrueColor) ||
+        (visual_info->klass == DirectColor))
       (void) SetImageType(image_list[scene],TrueColorType);
     if ((display_image->columns < image_list[scene]->columns) &&
         (display_image->rows < image_list[scene]->rows))
       display_image=image_list[scene];
   }
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     {
       (void) LogMagickEvent(X11Event,GetMagickModule(),
         "Image: %.1024s[%lu] %lux%lu ",display_image->filename,
@@ -1443,7 +1447,7 @@ MagickXAnimateImages(Display *display,
   manager_hints->initial_state=WithdrawnState;
   MagickXMakeWindow(display,root_window,argv,argc,class_hints,manager_hints,
     &windows->context);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),
       "Window id: 0x%lx (context)",windows->context.id);
   context_values.background=pixel->background_color.pixel;
@@ -1494,7 +1498,7 @@ MagickXAnimateImages(Display *display,
   manager_hints->initial_state=IconicState;
   MagickXMakeWindow(display,root_window,argv,argc,class_hints,manager_hints,
     &windows->icon);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (icon)",
       windows->icon.id);
   /*
@@ -1590,7 +1594,7 @@ MagickXAnimateImages(Display *display,
         resource_info->iconic ? IconicState : NormalState;
       MagickXMakeWindow(display,root_window,argv,argc,class_hints,manager_hints,
         &windows->backdrop);
-      if (IsEventLogging())
+      if (IsEventLogged(X11Event))
         (void) LogMagickEvent(X11Event,GetMagickModule(),
           "Window id: 0x%lx (backdrop)",windows->backdrop.id);
       (void) XMapWindow(display,windows->backdrop.id);
@@ -1626,7 +1630,7 @@ MagickXAnimateImages(Display *display,
       manager_hints->flags|=(unsigned int) WindowGroupHint;
       manager_hints->window_group=windows->group_leader.id;
       (void) XSelectInput(display,windows->group_leader.id,StructureNotifyMask);
-      if (IsEventLogging())
+      if (IsEventLogged(X11Event))
         (void) LogMagickEvent(X11Event,GetMagickModule(),
           "Window id: 0x%lx (group leader)",windows->group_leader.id);
     }
@@ -1638,7 +1642,7 @@ MagickXAnimateImages(Display *display,
   if (windows->group_leader.id != (Window) NULL)
     (void) XSetTransientForHint(display,windows->image.id,
       windows->group_leader.id);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (image)",
       windows->image.id);
   /*
@@ -1669,7 +1673,7 @@ MagickXAnimateImages(Display *display,
   (void) XSetTransientForHint(display,windows->info.id,windows->image.id);
   if (windows->image.mapped)
     (void) XWithdrawWindow(display,windows->info.id,windows->info.screen);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (info)",
       windows->info.id);
   /*
@@ -1698,7 +1702,7 @@ MagickXAnimateImages(Display *display,
   windows->command.highlight_stipple=windows->info.highlight_stipple;
   windows->command.shadow_stipple=windows->info.shadow_stipple;
   (void) XSetTransientForHint(display,windows->command.id,windows->image.id);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (command)",
       windows->command.id);
   /*
@@ -1731,7 +1735,7 @@ MagickXAnimateImages(Display *display,
   windows->widget.highlight_stipple=windows->info.highlight_stipple;
   windows->widget.shadow_stipple=windows->info.shadow_stipple;
   (void) XSetTransientForHint(display,windows->widget.id,windows->image.id);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (widget)",
       windows->widget.id);
   /*
@@ -1760,7 +1764,7 @@ MagickXAnimateImages(Display *display,
   windows->popup.highlight_stipple=windows->info.highlight_stipple;
   windows->popup.shadow_stipple=windows->info.shadow_stipple;
   (void) XSetTransientForHint(display,windows->popup.id,windows->image.id);
-  if (IsEventLogging())
+  if (IsEventLogged(X11Event))
     (void) LogMagickEvent(X11Event,GetMagickModule(),"Window id: 0x%lx (pop up)",
       windows->popup.id);
   if (!windows->image.mapped || (windows->backdrop.id != (Window) NULL))
@@ -1768,8 +1772,11 @@ MagickXAnimateImages(Display *display,
   /*
     Set out progress and warning handlers.
   */
-  if (monitor_handler == (MonitorHandler) NULL)
+#if defined(MAGICK_USE_XMAGICK_MONITOR) && MAGICK_USE_XMAGICK_MONITOR
+  if ((resource_info->image_info->progress)/*  && */
+      /* (monitor_handler == (MonitorHandler) NULL) */)
     monitor_handler=SetMonitorHandler(MagickXMagickMonitor);
+#endif /* if defined(MAGICK_USE_XMAGICK_MONITOR) && MAGICK_USE_XMAGICK_MONITOR */
   if (warning_handler == (WarningHandler) NULL)
     {
       warning_handler=resource_info->display_warnings ?
@@ -1811,8 +1818,8 @@ MagickXAnimateImages(Display *display,
     windows->image.pixmap=(Pixmap) NULL;
     windows->image.matte_pixmap=(Pixmap) NULL;
     if ((resource_info->map_type != (char *) NULL) ||
-        (visual_info->class == TrueColor) ||
-        (visual_info->class == DirectColor))
+        (visual_info->klass == TrueColor) ||
+        (visual_info->klass == DirectColor))
       if (image_list[scene]->storage_class == PseudoClass)
         {
           /*
@@ -1827,7 +1834,7 @@ MagickXAnimateImages(Display *display,
       (unsigned int) image_list[scene]->rows);
     if (status == False)
       MagickFatalError(XServerFatalError,UnableToCreateXImage,(char *) NULL);
-    if (IsEventLogging())
+    if (IsEventLogged(X11Event))
       {
         (void) LogMagickEvent(X11Event,GetMagickModule(),
           "Image: [%lu] %.1024s %lux%lu ",image_list[scene]->scene,
@@ -1987,7 +1994,7 @@ MagickXAnimateImages(Display *display,
             Copy X pixmap to Image window.
           */
           MagickXGetPixelPacket(display,visual_info,map_info,resource_info,
-            image_list[scene],&scene_info); /* FIXME leak here */
+            image_list[scene],&scene_info);
           windows->image.pixel_info=(&scene_info);
           windows->image.ximage->width=(unsigned int) image->columns;
           windows->image.ximage->height=(unsigned int) image->rows;
@@ -2048,7 +2055,7 @@ MagickXAnimateImages(Display *display,
     {
       case ButtonPress:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Button Press: 0x%lx %u +%d+%d",event.xbutton.window,
             event.xbutton.button,event.xbutton.x,event.xbutton.y);
@@ -2091,7 +2098,7 @@ MagickXAnimateImages(Display *display,
       }
       case ButtonRelease:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Button Release: 0x%lx %u +%d+%d",event.xbutton.window,
             event.xbutton.button,event.xbutton.x,event.xbutton.y);
@@ -2099,7 +2106,7 @@ MagickXAnimateImages(Display *display,
       }
       case ClientMessage:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Client Message: 0x%lx 0x%lx %d 0x%lx",event.xclient.window,
             event.xclient.message_type,event.xclient.format,(unsigned long)
@@ -2226,7 +2233,7 @@ MagickXAnimateImages(Display *display,
       }
       case ConfigureNotify:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Configure Notify: 0x%lx %dx%d+%d+%d %d",event.xconfigure.window,
             event.xconfigure.width,event.xconfigure.height,event.xconfigure.x,
@@ -2290,7 +2297,7 @@ MagickXAnimateImages(Display *display,
         /*
           Group leader has exited.
         */
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),"Destroy Notify: 0x%lx",
             event.xdestroywindow.window);
         if (event.xdestroywindow.window == windows->group_leader.id)
@@ -2316,7 +2323,7 @@ MagickXAnimateImages(Display *display,
       }
       case Expose:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Expose: 0x%lx %dx%d+%d+%d",event.xexpose.window,
             event.xexpose.width,event.xexpose.height,event.xexpose.x,
@@ -2350,7 +2357,7 @@ MagickXAnimateImages(Display *display,
         length=XLookupString((XKeyEvent *) &event.xkey,command,sizeof(command),
           &key_symbol,(XComposeStatus *) NULL);
         *(command+length)='\0';
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),"Key press: 0x%lx (%c)",
             key_symbol,*command);
         command_type=NullCommand;
@@ -2421,7 +2428,7 @@ MagickXAnimateImages(Display *display,
         */
         (void) XLookupString((XKeyEvent *) &event.xkey,command,sizeof(command),
           &key_symbol,(XComposeStatus *) NULL);
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Key release: 0x%lx (%c)",key_symbol,*command);
         break;
@@ -2442,7 +2449,7 @@ MagickXAnimateImages(Display *display,
       }
       case MapNotify:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),"Map Notify: 0x%lx",
             event.xmap.window);
         if (event.xmap.window == windows->backdrop.id)
@@ -2528,7 +2535,7 @@ MagickXAnimateImages(Display *display,
           after,
           length;
 
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Property Notify: 0x%lx 0x%lx %d",event.xproperty.window,
             event.xproperty.atom,event.xproperty.state);
@@ -2555,7 +2562,7 @@ MagickXAnimateImages(Display *display,
       }
       case ReparentNotify:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),
             "Reparent Notify: 0x%lx=>0x%lx",event.xreparent.parent,
             event.xreparent.window);
@@ -2563,7 +2570,7 @@ MagickXAnimateImages(Display *display,
       }
       case UnmapNotify:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),"Unmap Notify: 0x%lx",
             event.xunmap.window);
         if (event.xunmap.window == windows->backdrop.id)
@@ -2616,7 +2623,7 @@ MagickXAnimateImages(Display *display,
       }
       default:
       {
-        if (IsEventLogging())
+        if (IsEventLogged(X11Event))
           (void) LogMagickEvent(X11Event,GetMagickModule(),"Event type: %d",
             event.type);
         break;
@@ -2630,9 +2637,9 @@ MagickXAnimateImages(Display *display,
       DestroyImageList(images);
       images=(Image *) NULL;
     }
-  if ((windows->visual_info->class == GrayScale) ||
-      (windows->visual_info->class == PseudoColor) ||
-      (windows->visual_info->class == DirectColor))
+  if ((windows->visual_info->klass == GrayScale) ||
+      (windows->visual_info->klass == PseudoColor) ||
+      (windows->visual_info->klass == DirectColor))
     {
       /*
         Withdraw windows.
@@ -2668,6 +2675,7 @@ MagickXAnimateImages(Display *display,
   }
   MagickFreeMemory(windows->image.pixmaps);
   MagickFreeMemory(windows->image.matte_pixmaps);
+  MagickFreeMemory(scene_info.pixels);
   if (nexus == (Image *) NULL)
     {
       /*
