@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003-2026 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -27,21 +27,50 @@
 */
 #define ColorFilename  "colors.mgk"
 
+
+/*
+  Information about a named color
+*/
+typedef struct _ColorInfo
+{
+  char
+    *path,
+    *name;
+
+  ComplianceType
+    compliance;
+
+  PixelPacket
+    color;
+
+  unsigned int
+    stealth;
+
+  unsigned long
+    signature;
+
+  struct _ColorInfo
+    *previous,
+    *next;
+} ColorInfo;
+
+static const char
+  BuiltInPath[]="[Built In]";
+
 /*
   Declare color map.
 */
-
-static const char
-  *BuiltInPath="[Built In]";
-
 static const struct
 {
-  char
-    *name;
+  /* Color name */
+  const char
+    name[21];
 
+  /* Compliance based on flags from ComplianceType enumeration */
   unsigned short
     compliance;
 
+  /* Color value */
   unsigned char
     red,
     green,
@@ -631,12 +660,22 @@ static SemaphoreInfo
 
 static ColorInfo
   *color_list = (ColorInfo *) NULL;
+
 
 /*
   Forward declarations.
 */
 static unsigned int
   ReadColorConfigureFile(const char *,const unsigned int,ExceptionInfo *);
+
+static const ColorInfo
+  *GetColorInfo(const char *name, ExceptionInfo *exception);
+
+#if defined(EMIT_GET_COLOR_INFO_ARRAY)
+static ColorInfo
+  **GetColorInfoArray(ExceptionInfo *exception);
+#endif
+
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -672,9 +711,9 @@ DestroyColorInfoEntry(ColorInfo *entry)
       MagickFreeMemory(entry->path);
       MagickFreeMemory(entry->name);
     }
-  MagickFreeMemory(entry);    
+  MagickFreeMemory(entry);
 }
-MagickExport void
+void
 DestroyColorInfo(void)
 {
   ColorInfo
@@ -704,8 +743,12 @@ DestroyColorInfo(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method GetColorInfo searches the color list for the specified name and if
-%  found returns attributes for that color.
+%  Method GetColorInfo uses case-insensitive matching to search the color
+%  list for the specified NUL-terminated name and if found returns
+%  attributes for that color.  If the name argument is NULL, or is the
+%  character '*', then return the head of the ColorInfo doubly-linked list.
+%
+%  The maximum color name length compared is 64 characters.
 %
 %  The format of the GetColorInfo method is:
 %
@@ -725,69 +768,71 @@ DestroyColorInfo(void)
 %
 %
 */
-MagickExport const ColorInfo *
+static const ColorInfo *
 GetColorInfo(const char *name,ExceptionInfo *exception)
 {
-  char
-    colorname[MaxTextExtent];
+#define MaxColorLength 64
 
   register ColorInfo
     *p;
 
-  LockSemaphoreInfo(color_semaphore);
+  /*
+    If color configuration file (which includes the static table) has
+    not been loaded, then load it.
+  */
   if (color_list == (ColorInfo *) NULL)
     (void) ReadColorConfigureFile(ColorFilename,0,exception);
-  UnlockSemaphoreInfo(color_semaphore);
-  if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
+  /*
+    If name is null, or '*', then return the head of the list
+  */
+  if ((name == (const char *) NULL) ||
+      ((name[0] == '*') && (name[1] == '\0')))
     return((const ColorInfo *) color_list);
   /*
-    Search for named color.
+    If name is empty, then stop looking
   */
-  if (strlcpy(colorname,name,MaxTextExtent) >= MaxTextExtent)
+  if (name[0] == '\0')
     {
       ThrowException(exception,OptionWarning,UnrecognizedColor,name);
       return (const ColorInfo *) NULL;
     }
   /*
-    Compact by removing spaces
+    Search for named color.
   */
-  LockSemaphoreInfo(color_semaphore);
   for (p=color_list; p != (ColorInfo *) NULL; p=p->next)
-    if (LocaleCompare(colorname,p->name) == 0)
+    if (LocaleNCompare(name,p->name,MaxColorLength) == 0)
       break;
   if (p == (ColorInfo *) NULL)
     {
-      /* Check common synonyms */
-      const char
-	*pos;
+      /*
+        Check common synonym for gray
+      */
+      char
+        colorname[MaxColorLength+1];
 
-      LocaleUpper(colorname);
-      if ((pos = strstr(colorname,"GREY")) != (const char *) NULL)
-	{
-	  colorname[pos-colorname+2]='A';
-	  for (p=color_list; p != (ColorInfo *) NULL; p=p->next)
-	    if (LocaleCompare(colorname,p->name) == 0)
-	      break;
-	}
+      if (strlcpy(colorname,name,sizeof(colorname)) >= sizeof(colorname))
+        {
+          ThrowException(exception,OptionWarning,UnrecognizedColor,name);
+        }
+      else
+        {
+          const char
+            *pos;
+
+          LocaleUpper(colorname);
+          if ((pos = strstr(colorname,"GREY")) != (const char *) NULL)
+            {
+              colorname[pos-colorname+2]='A';
+              for (p=color_list; p != (ColorInfo *) NULL; p=p->next)
+                if (LocaleNCompare(colorname,p->name,MaxColorLength) == 0)
+                  break;
+            }
+        }
     }
   if (p == (ColorInfo *) NULL)
-    ThrowException(exception,OptionWarning,UnrecognizedColor,name);
-  else
-    if (p != color_list)
-      {
-        /*
-          Self-adjusting list.
-        */
-        if (p->previous != (ColorInfo *) NULL)
-          p->previous->next=p->next;
-        if (p->next != (ColorInfo *) NULL)
-          p->next->previous=p->previous;
-        p->previous=(ColorInfo *) NULL;
-        p->next=color_list;
-        color_list->previous=p;
-        color_list=p;
-      }
-  UnlockSemaphoreInfo(color_semaphore);
+    {
+      ThrowException(exception,OptionWarning,UnrecognizedColor,name);
+    }
   return ((const ColorInfo *) p);
 }
 
@@ -796,7 +841,7 @@ GetColorInfo(const char *name,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   G e t C o l o r I n f o A r r a y                                         %
++   G e t C o l o r I n f o A r r a y                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -823,6 +868,7 @@ GetColorInfo(const char *name,ExceptionInfo *exception)
 /*
   Compare two ColorInfo structures based on their name
 */
+#if defined(EMIT_GET_COLOR_INFO_ARRAY)
 static int
 ColorInfoCompare(const void *x, const void *y)
 {
@@ -832,7 +878,7 @@ ColorInfoCompare(const void *x, const void *y)
 
   return strcmp(xx->name, yy->name);
 }
-MagickExport ColorInfo **
+static ColorInfo **
 GetColorInfoArray(ExceptionInfo *exception)
 {
   ColorInfo
@@ -857,8 +903,6 @@ GetColorInfoArray(ExceptionInfo *exception)
   if ((!color_list) || (exception->severity > UndefinedException))
     return 0;
 
-  LockSemaphoreInfo(color_semaphore);
-
   list=color_list;
 
   /*
@@ -873,7 +917,6 @@ GetColorInfoArray(ExceptionInfo *exception)
   array=MagickAllocateMemory(ColorInfo **,sizeof(ColorInfo *)*(entries+1));
   if (!array)
     {
-      UnlockSemaphoreInfo(color_semaphore);
       ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,0);
       return False;
     }
@@ -886,8 +929,6 @@ GetColorInfoArray(ExceptionInfo *exception)
   for (p=list; p != 0; p=p->next)
     array[i++]=p;
 
-  UnlockSemaphoreInfo(color_semaphore);
-
   /*
     Sort array entries
   */
@@ -895,6 +936,7 @@ GetColorInfoArray(ExceptionInfo *exception)
 
   return (array);
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -939,7 +981,6 @@ GetColorList(const char *pattern,unsigned long *number_colors)
   register const ColorInfo
     *p;
 
-
   assert(pattern != (char *) NULL);
   assert(number_colors != (unsigned long *) NULL);
   *number_colors=0;
@@ -952,32 +993,32 @@ GetColorList(const char *pattern,unsigned long *number_colors)
   /*
     Determine color list size
   */
-  LockSemaphoreInfo(color_semaphore);
-  i=0;
-  for (p=color_list; p != (const ColorInfo *) NULL; p=p->next)
-    i++;
-  UnlockSemaphoreInfo(color_semaphore);
+  while(1)
+    {
+      i=0;
+      for (p=color_list; p != (const ColorInfo *) NULL; p=p->next)
+        i++;
 
-  /*
-    Allocate color list.
-  */
-  colorlist=MagickAllocateMemory(char **,i*sizeof(char *));
-  if (colorlist == (char **) NULL)
-    return((char **) NULL);
+      /*
+        Allocate color list.
+      */
+      colorlist=MagickAllocateMemory(char **,i*sizeof(char *));
+      if (colorlist == (char **) NULL)
+        break;
 
-  /*
-    Add colors matching glob specification to list
-  */
-  LockSemaphoreInfo(color_semaphore);
-  i=0;
-  for (p=color_list; p != (const ColorInfo *) NULL; p=p->next)
-  {
-    if (p->stealth)
-      continue;
-    if (GlobExpression(p->name,pattern))
-      colorlist[i++]=AcquireString(p->name);
-  }
-  UnlockSemaphoreInfo(color_semaphore);
+      /*
+        Add colors matching glob specification to list
+      */
+      i=0;
+      for (p=color_list; p != (const ColorInfo *) NULL; p=p->next)
+        {
+          if (p->stealth)
+            continue;
+          if (GlobExpression(p->name,pattern))
+            colorlist[i++]=AcquireString(p->name);
+        }
+      break;
+    };
 
   *number_colors=i;
   return(colorlist);
@@ -1010,7 +1051,7 @@ GetColorList(const char *pattern,unsigned long *number_colors)
 %
 %
 */
-MagickExport unsigned int
+unsigned int
 ListColorInfo(FILE *file,ExceptionInfo *exception)
 {
   register long
@@ -1022,7 +1063,6 @@ ListColorInfo(FILE *file,ExceptionInfo *exception)
   if (file == (const FILE *) NULL)
     file=stdout;
   (void) GetColorInfo("*",exception);
-  LockSemaphoreInfo(color_semaphore);
   for (p=color_list; p != (const ColorInfo *) NULL; p=p->next)
   {
     if ((p->previous == (ColorInfo *) NULL) ||
@@ -1046,15 +1086,15 @@ ListColorInfo(FILE *file,ExceptionInfo *exception)
 
     if (p->color.opacity == OpaqueOpacity)
       (void) fprintf(file,"%5d,%5d,%5d       ",
-		     ScaleQuantumToChar(p->color.red),
-		     ScaleQuantumToChar(p->color.green),
-		     ScaleQuantumToChar(p->color.blue));
+                     ScaleQuantumToChar(p->color.red),
+                     ScaleQuantumToChar(p->color.green),
+                     ScaleQuantumToChar(p->color.blue));
     else
       (void) fprintf(file,"%5d,%5d,%5d,%5d ",
-		     ScaleQuantumToChar(p->color.red),
-		     ScaleQuantumToChar(p->color.green),
-		     ScaleQuantumToChar(p->color.blue),
-		     ScaleQuantumToChar(p->color.opacity));
+                     ScaleQuantumToChar(p->color.red),
+                     ScaleQuantumToChar(p->color.green),
+                     ScaleQuantumToChar(p->color.blue),
+                     ScaleQuantumToChar(p->color.opacity));
     if ((unsigned int) p->compliance & (unsigned int) SVGCompliance)
       (void) fprintf(file,"SVG ");
     if ((unsigned int) p->compliance & (unsigned int) X11Compliance)
@@ -1064,7 +1104,6 @@ ListColorInfo(FILE *file,ExceptionInfo *exception)
     (void) fprintf(file,"\n");
   }
   (void) fflush(file);
-  UnlockSemaphoreInfo(color_semaphore);
   return(True);
 }
 
@@ -1107,7 +1146,10 @@ InitializeColorInfo(void)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  QueryColorDatabase() returns the red, green, blue, and opacity intensities
-%  for a given color name.
+%  for a given color name.  The color specification syntax is a superset of
+%  what is specified in the CSS Color Module Level 3
+% (https://www.w3.org/TR/css-color-3/#rgb-color), which uses the same syntax
+%  as SVG2.
 %
 %  The format of the QueryColorDatabase method is:
 %
@@ -1127,7 +1169,7 @@ InitializeColorInfo(void)
 */
 MagickExport unsigned int
 QueryColorDatabase(const char *name,PixelPacket *color,
-		   ExceptionInfo *exception)
+                   ExceptionInfo *exception)
 {
   double
     scale;
@@ -1234,7 +1276,7 @@ QueryColorDatabase(const char *name,PixelPacket *color,
       {
         unsigned int
           divisor=1;
-        
+
         n<<=2;
         for( i=n-1; i; i--)
           {
@@ -1257,31 +1299,61 @@ QueryColorDatabase(const char *name,PixelPacket *color,
   if (LocaleNCompare(name,"rgb(",4) == 0)
     {
       DoublePixelPacket
-	pixel;
+        pixel;
+
+      int
+        count;
+
+      /*
+        FIXME: Deviates from CSS-3 specification
+
+        CSS-3 uses RGB integer range 0-255, if not percentage.
+      */
 
       scale=strchr(name,'%') == (char *) NULL ? 1.0 :
         ScaleQuantumToChar(MaxRGB)/100.0;
-      (void) sscanf(name,"%*[^(](%lf%*[%,]%lf%*[%,]%lf",
+      pixel.red=pixel.green=pixel.blue=0.0;
+      count=sscanf(name,"%*[^(](%lf%*[%,]%lf%*[%,]%lf",
         &pixel.red,&pixel.green,&pixel.blue);
-      color->red=ScaleCharToQuantum(scale*pixel.red);
-      color->green=ScaleCharToQuantum(scale*pixel.green);
-      color->blue=ScaleCharToQuantum(scale*pixel.blue);
+      if (count > 0)
+        color->red=ScaleCharToQuantum(scale*pixel.red);
+      if (count > 1)
+        color->green=ScaleCharToQuantum(scale*pixel.green);
+      if (count > 2)
+        color->blue=ScaleCharToQuantum(scale*pixel.blue);
       color->opacity=OpaqueOpacity;
       return(True);
     }
   if (LocaleNCompare(name,"rgba(",5) == 0)
     {
       DoublePixelPacket
-	pixel;
+        pixel;
+
+      int
+        count;
+
+      /*
+        FIXME: Deviates from CSS-3 specification
+
+        CSS-3 uses RGB integer range 0-255, if not percentage.
+        CSS-3 uses non-percent decimal values for Alpha.
+        CSS-3 uses a value 0 to mean totally transparent.
+      */
 
       scale=strchr(name,'%') == (char *) NULL ? 1.0 :
         ScaleQuantumToChar(MaxRGB)/100.0;
-      (void) sscanf(name,"%*[^(](%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf",
+      pixel.red=pixel.green=pixel.blue=0.0;
+      color->opacity=OpaqueOpacity;
+      count = sscanf(name,"%*[^(](%lf%*[%,]%lf%*[%,]%lf%*[%,]%lf",
         &pixel.red,&pixel.green,&pixel.blue,&pixel.opacity);
-      color->red=ScaleCharToQuantum(scale*pixel.red);
-      color->green=ScaleCharToQuantum(scale*pixel.green);
-      color->blue=ScaleCharToQuantum(scale*pixel.blue);
-      color->opacity=ScaleCharToQuantum(scale*pixel.opacity);
+      if (count > 0)
+        color->red=ScaleCharToQuantum(scale*pixel.red);
+      if (count > 1)
+        color->green=ScaleCharToQuantum(scale*pixel.green);
+      if (count > 2)
+        color->blue=ScaleCharToQuantum(scale*pixel.blue);
+      if (count > 3)
+        color->opacity=ScaleCharToQuantum(scale*pixel.opacity);
       return(True);
     }
   p=GetColorInfo(name,exception);
@@ -1309,7 +1381,7 @@ QueryColorDatabase(const char *name,PixelPacket *color,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  QueryColorname() returns a named color for the given color intensity.  If
-%  an exact match is not found, a hex value is return instead.  For example
+%  an exact match is not found, a hex value is returned instead.  For example
 %  an intensity of rgb:(0,0,0) returns black whereas rgb:(223,223,223)
 %  returns #dfdfdf.
 %
@@ -1336,8 +1408,8 @@ QueryColorDatabase(const char *name,PixelPacket *color,
 */
 MagickExport unsigned int
 QueryColorname(const Image *image,const PixelPacket *color,
-	       const ComplianceType compliance,char *name,
-	       ExceptionInfo *exception)
+               const ComplianceType compliance,char *name,
+               ExceptionInfo *exception)
 {
   register const ColorInfo
     *p;
@@ -1402,8 +1474,8 @@ QueryColorname(const Image *image,const PixelPacket *color,
 */
 static MagickPassFail
 ReadColorConfigureFile(const char *basename,
-		       const unsigned int depth,
-		       ExceptionInfo *exception)
+                       const unsigned int depth,
+                       ExceptionInfo *exception)
 {
   size_t
     length;
@@ -1412,43 +1484,53 @@ ReadColorConfigureFile(const char *basename,
     path[MaxTextExtent],
     *xml;
 
+  MagickPassFail
+    status=MagickPass;
+
+  LockSemaphoreInfo(color_semaphore);
+  if (color_list != (ColorInfo *) NULL)
+    {
+      UnlockSemaphoreInfo(color_semaphore);
+      return status;
+    }
+
   if (depth == 0)
     {
       size_t
-	i;
+        i;
 
       /*
-	Load default set of colors from the static color table.
+        Load default set of colors from the static color table.
       */
       for (i=0 ; i < sizeof(StaticColors)/sizeof(StaticColors[0]); i++)
-	{
-	  ColorInfo
-	    *color_info;
+        {
+          ColorInfo
+            *color_info;
 
-	  color_info=MagickAllocateMemory(ColorInfo *,sizeof(ColorInfo));
-	  if (color_info == (ColorInfo *) NULL)
-	    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-			      UnableToAllocateColorInfo);
-	  color_info->path=(char *) BuiltInPath;
-	  color_info->name=(char *) StaticColors[i].name;
-	  color_info->compliance=(ComplianceType) StaticColors[i].compliance;
-	  color_info->color.red=ScaleCharToQuantum(StaticColors[i].red);
-	  color_info->color.green=ScaleCharToQuantum(StaticColors[i].green);
-	  color_info->color.blue=ScaleCharToQuantum(StaticColors[i].blue);
-	  color_info->color.opacity=ScaleCharToQuantum(StaticColors[i].opacity);
-	  color_info->stealth=MagickFalse;
-	  color_info->signature=MagickSignature;
-	  color_info->previous=(ColorInfo *) NULL;
-	  color_info->next=(ColorInfo *) NULL;
-	  if (color_list == (ColorInfo *) NULL)
-	    {
-	      color_list=color_info;
-	      continue;
-	    }
-	  color_list->next=color_info;
-	  color_info->previous=color_list;
-	  color_list=color_list->next;
-	}
+          color_info=MagickAllocateMemory(ColorInfo *,sizeof(ColorInfo));
+          if (color_info == (ColorInfo *) NULL)
+            MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                              UnableToAllocateColorInfo);
+          color_info->path=(char *) BuiltInPath;
+          color_info->name=(char *) StaticColors[i].name;
+          color_info->compliance=(ComplianceType) StaticColors[i].compliance;
+          color_info->color.red=ScaleCharToQuantum(StaticColors[i].red);
+          color_info->color.green=ScaleCharToQuantum(StaticColors[i].green);
+          color_info->color.blue=ScaleCharToQuantum(StaticColors[i].blue);
+          color_info->color.opacity=ScaleCharToQuantum(StaticColors[i].opacity);
+          color_info->stealth=MagickFalse;
+          color_info->signature=MagickSignature;
+          color_info->previous=(ColorInfo *) NULL;
+          color_info->next=(ColorInfo *) NULL;
+          if (color_list == (ColorInfo *) NULL)
+            {
+              color_list=color_info;
+              continue;
+            }
+          color_list->next=color_info;
+          color_info->previous=color_list;
+          color_list=color_list->next;
+        }
     }
 
   /*
@@ -1458,12 +1540,12 @@ ReadColorConfigureFile(const char *basename,
   if (depth == 0)
     {
       ExceptionInfo
-	exception_local;
+        exception_local;
 
       GetExceptionInfo(&exception_local);
       xml=(char *) GetConfigureBlob(basename,path,&length,&exception_local);
       if (exception_local.severity != ConfigureError)
-	CopyException(exception,&exception_local);
+        CopyException(exception,&exception_local);
       DestroyExceptionInfo(&exception_local);
     }
   else
@@ -1473,224 +1555,232 @@ ReadColorConfigureFile(const char *basename,
   if (xml != (char *) NULL)
     {
       char
-	keyword[MaxTextExtent],
-	*q,
-	*token;
+        keyword[MaxTextExtent],
+        *q,
+        *token;
+
+      size_t
+        token_max_length;
 
       MagickBool
-	in_entry;
+        in_entry;
 
       token=AcquireString(xml);
+      token_max_length=strlen(token);
       in_entry=MagickFalse;
       for (q=xml; *q != '\0'; )
-	{
-	  /*
-	    Interpret XML.
-	  */
-	  GetToken(q,&q,token);
-	  if (*token == '\0')
-	    break;
-	  (void) strlcpy(keyword,token,sizeof(keyword));
-	  if (LocaleNCompare(keyword,"<!--",4) == 0)
-	    {
-	      /*
-		Comment element.
-	      */
-	      while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-		GetToken(q,&q,token);
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"<include") == 0)
-	    {
-	      /*
-		Include element.
-	      */
-	      while ((*token != '>') && (*q != '\0'))
-		{
-		  (void) strlcpy(keyword,token,MaxTextExtent);
-		  GetToken(q,&q,token);
-		  if (*token != '=')
-		    continue;
-		  GetToken(q,&q,token);
-		  if (LocaleCompare(keyword,"file") == 0)
-		    {
-		      if (depth > 200)
-			ThrowException(exception,ConfigureError,
-				       IncludeElementNestedTooDeeply,path);
-		      else
-			{
-			  char
-			    filename[MaxTextExtent];
-
-			  GetPathComponent(path,HeadPath,filename);
-			  if (*filename != '\0')
-			    (void) strlcat(filename,DirectorySeparator,sizeof(filename));
-			  (void) strlcat(filename,token,sizeof(filename));
-			  (void) ReadColorConfigureFile(filename,depth+1,exception);
-			}
-		      if (color_list != (ColorInfo *) NULL)
-			while (color_list->next != (ColorInfo *) NULL)
-			  color_list=color_list->next;
-		    }
-		}
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"<color") == 0)
-	    {
-	      ColorInfo
-		*color_info;
-
-	      /*
-		Allocate memory for the color list.
-	      */
-	      in_entry=MagickTrue;
-	      color_info=MagickAllocateMemory(ColorInfo *,sizeof(ColorInfo));
-	      if (color_info == (ColorInfo *) NULL)
-		MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-				  UnableToAllocateColorInfo);
-	      (void) memset(color_info,0,sizeof(ColorInfo));
-	      color_info->path=AcquireString(path);
-	      color_info->signature=MagickSignature;
-	      if (color_list == (ColorInfo *) NULL)
-		{
-		  color_list=color_info;
-		  continue;
-		}
-	      color_list->next=color_info;
-	      color_info->previous=color_list;
-	      color_list=color_list->next;
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"/>") == 0)
-	    {
-	      /*
-		Closing a color specification.
-	      */
-	      if (in_entry)
-		{
-		  /*
-		    Remove any existing entry with same name (last one wins).
-		  */
-		  {
-		    ColorInfo
-		      *color_info;
-	      
-		    for (color_info=color_list->previous;
-			 color_info != (ColorInfo *) NULL;
-			 color_info=color_info->previous)
-		      {
-			if ((LocaleCompare(color_list->name,color_info->name) == 0) &&
-			    (color_list->compliance == color_info->compliance))
-			  {
-			    DestroyColorInfoEntry(color_info);
-			    break;
-			  }
-		      }
-		  }
-		  in_entry=MagickFalse;
-		}
-	    }
-	  if (color_list == (ColorInfo *) NULL)
-	    continue;
-	  GetToken(q,(char **) NULL,token);
-          if (token == (char *) NULL)
+        {
+          /*
+            Interpret XML.
+          */
+          MagickGetToken(q,&q,token,token_max_length);
+          if (*token == '\0')
             break;
-	  if (*token != '=')
-	    continue;
-	  GetToken(q,&q,token);
-	  GetToken(q,&q,token);
-	  switch (*keyword)
-	    {
-	    case 'B':
-	    case 'b':
-	      {
-		if (LocaleCompare((char *) keyword,"blue") == 0)
-		  {
-		    color_list->color.blue=ScaleCharToQuantum(MagickAtoL(token));
-		    break;
-		  }
-		break;
-	      }
-	    case 'C':
-	    case 'c':
-	      {
-		if (LocaleCompare((char *) keyword,"compliance") == 0)
-		  {
-		    long
-		      compliance;
+          (void) strlcpy(keyword,token,sizeof(keyword));
+          if (LocaleNCompare(keyword,"<!--",4) == 0)
+            {
+              /*
+                Comment element.
+              */
+              while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+                MagickGetToken(q,&q,token,token_max_length);
+              continue;
+            }
+          if (LocaleCompare(keyword,"<include") == 0)
+            {
+              /*
+                Include element.
+              */
+              while ((*token != '>') && (*q != '\0'))
+                {
+                  (void) strlcpy(keyword,token,MaxTextExtent);
+                  MagickGetToken(q,&q,token,token_max_length);
+                  if (*token != '=')
+                    continue;
+                  MagickGetToken(q,&q,token,token_max_length);
+                  if (LocaleCompare(keyword,"file") == 0)
+                    {
+                      if (depth > 200)
+                        ThrowException(exception,ConfigureError,
+                                       IncludeElementNestedTooDeeply,path);
+                      else
+                        {
+                          char
+                            filename[MaxTextExtent];
 
-		    compliance=color_list->compliance;
-		    if (GlobExpression(token,"*SVG*"))
-		      compliance|=SVGCompliance;
-		    if (GlobExpression(token,"*X11*"))
-		      compliance|=X11Compliance;
-		    if (GlobExpression(token,"*XPM*"))
-		      compliance|=XPMCompliance;
-		    color_list->compliance=(ComplianceType) compliance;
-		    break;
-		  }
-		break;
-	      }
-	    case 'G':
-	    case 'g':
-	      {
-		if (LocaleCompare((char *) keyword,"green") == 0)
-		  {
-		    color_list->color.green=ScaleCharToQuantum(MagickAtoL(token));
-		    break;
-		  }
-		break;
-	      }
-	    case 'N':
-	    case 'n':
-	      {
-		if (LocaleCompare((char *) keyword,"name") == 0)
-		  {
-		    color_list->name=AcquireString(token);
-		    break;
-		  }
-		break;
-	      }
-	    case 'O':
-	    case 'o':
-	      {
-		if (LocaleCompare((char *) keyword,"opacity") == 0)
-		  {
-		    color_list->color.opacity=ScaleCharToQuantum(MagickAtoL(token));
-		    break;
-		  }
-		break;
-	      }
-	    case 'R':
-	    case 'r':
-	      {
-		if (LocaleCompare((char *) keyword,"red") == 0)
-		  {
-		    color_list->color.red=ScaleCharToQuantum(MagickAtoL(token));
-		    break;
-		  }
-		break;
-	      }
-	    case 'S':
-	    case 's':
-	      {
-		if (LocaleCompare((char *) keyword,"stealth") == 0)
-		  {
-		    color_list->stealth=LocaleCompare(token,"True") == 0;
-		    break;
-		  }
-		break;
-	      }
-	    default:
-	      break;
-	    }
-	}
+                          GetPathComponent(path,HeadPath,filename);
+                          if (*filename != '\0')
+                            (void) strlcat(filename,DirectorySeparator,sizeof(filename));
+                          (void) strlcat(filename,token,sizeof(filename));
+                          (void) ReadColorConfigureFile(filename,depth+1,exception);
+                        }
+                      if (color_list != (ColorInfo *) NULL)
+                        while (color_list->next != (ColorInfo *) NULL)
+                          color_list=color_list->next;
+                    }
+                }
+              continue;
+            }
+          if (LocaleCompare(keyword,"<color") == 0)
+            {
+              ColorInfo
+                *color_info;
+
+              /*
+                Allocate memory for the color list.
+              */
+              in_entry=MagickTrue;
+              color_info=MagickAllocateMemory(ColorInfo *,sizeof(ColorInfo));
+              if (color_info == (ColorInfo *) NULL)
+                MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                                  UnableToAllocateColorInfo);
+              (void) memset(color_info,0,sizeof(ColorInfo));
+              color_info->path=AcquireString(path);
+              color_info->signature=MagickSignature;
+              if (color_list == (ColorInfo *) NULL)
+                {
+                  color_list=color_info;
+                  continue;
+                }
+              color_list->next=color_info;
+              color_info->previous=color_list;
+              color_list=color_list->next;
+              continue;
+            }
+          if (LocaleCompare(keyword,"/>") == 0)
+            {
+              /*
+                Closing a color specification.
+              */
+              if (in_entry)
+                {
+                  /*
+                    Remove any existing entry with same name (last one wins).
+                  */
+                  {
+                    ColorInfo
+                      *color_info;
+
+                    for (color_info=color_list->previous;
+                         color_info != (ColorInfo *) NULL;
+                         color_info=color_info->previous)
+                      {
+                        if ((LocaleCompare(color_list->name,color_info->name) == 0) &&
+                            (color_list->compliance == color_info->compliance))
+                          {
+                            DestroyColorInfoEntry(color_info);
+                            break;
+                          }
+                      }
+                  }
+                  in_entry=MagickFalse;
+                }
+            }
+          if (color_list == (ColorInfo *) NULL)
+            continue;
+          MagickGetToken(q,(char **) NULL,token,token_max_length);
+          if (*token != '=')
+            continue;
+          MagickGetToken(q,&q,token,token_max_length);
+          MagickGetToken(q,&q,token,token_max_length);
+          switch (*keyword)
+            {
+            case 'B':
+            case 'b':
+              {
+                if (LocaleCompare((char *) keyword,"blue") == 0)
+                  {
+                    color_list->color.blue=ScaleCharToQuantum(MagickAtoL(token));
+                    break;
+                  }
+                break;
+              }
+            case 'C':
+            case 'c':
+              {
+                if (LocaleCompare((char *) keyword,"compliance") == 0)
+                  {
+                    long
+                      compliance;
+
+                    compliance=color_list->compliance;
+                    if (GlobExpression(token,"*SVG*"))
+                      compliance|=SVGCompliance;
+                    if (GlobExpression(token,"*X11*"))
+                      compliance|=X11Compliance;
+                    if (GlobExpression(token,"*XPM*"))
+                      compliance|=XPMCompliance;
+                    color_list->compliance=(ComplianceType) compliance;
+                    break;
+                  }
+                break;
+              }
+            case 'G':
+            case 'g':
+              {
+                if (LocaleCompare((char *) keyword,"green") == 0)
+                  {
+                    color_list->color.green=ScaleCharToQuantum(MagickAtoL(token));
+                    break;
+                  }
+                break;
+              }
+            case 'N':
+            case 'n':
+              {
+                if (LocaleCompare((char *) keyword,"name") == 0)
+                  {
+                    color_list->name=AcquireString(token);
+                    break;
+                  }
+                break;
+              }
+            case 'O':
+            case 'o':
+              {
+                if (LocaleCompare((char *) keyword,"opacity") == 0)
+                  {
+                    color_list->color.opacity=ScaleCharToQuantum(MagickAtoL(token));
+                    break;
+                  }
+                break;
+              }
+            case 'R':
+            case 'r':
+              {
+                if (LocaleCompare((char *) keyword,"red") == 0)
+                  {
+                    color_list->color.red=ScaleCharToQuantum(MagickAtoL(token));
+                    break;
+                  }
+                break;
+              }
+            case 'S':
+            case 's':
+              {
+                if (LocaleCompare((char *) keyword,"stealth") == 0)
+                  {
+                    color_list->stealth=LocaleCompare(token,"True") == 0;
+                    break;
+                  }
+                break;
+              }
+            default:
+              break;
+            }
+        }
       MagickFreeMemory(token);
       MagickFreeMemory(xml);
     }
   if (color_list == (ColorInfo *) NULL)
-    return(MagickFail);
-  while (color_list->previous != (ColorInfo *) NULL)
-    color_list=color_list->previous;
-  return(MagickPass);
+    {
+      status=MagickFail;
+    }
+  else
+    {
+      while (color_list->previous != (ColorInfo *) NULL)
+        color_list=color_list->previous;
+    }
+   UnlockSemaphoreInfo(color_semaphore);
+  return(status);
 }
