@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2023 GraphicsMagick Group
+% Copyright (C) 2003-2024 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -525,10 +525,13 @@ GetMagickInfoArray(ExceptionInfo *exception)
     Load all modules and obtain pointer to head of list
   */
   (void) GetMagickInfo("*",exception);
-  if (!magick_list)
-    return ((MagickInfo **) NULL);
 
   LockSemaphoreInfo(magick_semaphore);
+  if (!magick_list)
+    {
+      UnlockSemaphoreInfo(magick_semaphore);
+      return ((MagickInfo **) NULL);
+    }
 
   list=magick_list;
 
@@ -541,7 +544,7 @@ GetMagickInfoArray(ExceptionInfo *exception)
   /*
     Allocate array memory
   */
-  array=MagickAllocateArray(MagickInfo **,sizeof(MagickInfo *),(entries+1));
+  array=MagickAllocateArray(MagickInfo **,(entries+1),sizeof(MagickInfo *));
   if (!array)
     {
       UnlockSemaphoreInfo(magick_semaphore);
@@ -1205,7 +1208,7 @@ InitializeMagickEx(const char *path, unsigned int options,
 
 #if defined(USE_GLIBC_MTRACE) && USE_GLIBC_MTRACE
   mtrace();
-#endif /* f defined(USE_GLIBC_MTRACE) && USE_GLIBC_MTRACE */
+#endif /* if defined(USE_GLIBC_MTRACE) && USE_GLIBC_MTRACE */
 
   /* Save initialization options in case we need them later */
   initialize_magick_options = options;
@@ -1562,9 +1565,10 @@ ListModuleMap(FILE *file,ExceptionInfo *exception)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Method MagickToMime returns the officially registered (or de facto) MIME
-%  media-type corresponding to a magick string.  If there is no registered
-%  media-type, then the string "image/x-magick" (all lower case) is returned.
-%  The returned string must be deallocated by the user.
+%  media-type (see https://www.iana.org/assignments/media-types/media-types.xhtml)
+%  corresponding to a magick string.  If there is no registered media-type,
+%  then the string "image/x-magick" (all lower case) is returned.  The returned
+%  string must be deallocated by the user.
 %
 %  The format of the MagickToMime method is:
 %
@@ -1588,17 +1592,20 @@ MagickToMime(const char *magick)
   static const struct
   {
     const char
-      magick[6],
+      magick[14],
       *media;
   }
   MediaTypes[] =
     {
-      { "avi",   "video/avi" },
       { "apng",  "image/apng" },
+      { "avc",   "image/avci" },
+      { "avi",   "video/avi" },
       { "avif",  "image/avif" },
       { "bmp",   "image/bmp" },
       { "cgm",   "image/cgm;Version=4;ProfileId=WebCGM" }, /* W3 WebCGM */
       { "dcm",   "application/dicom" }, /* Incomplete.  See RFC 3240 */
+      { "dpx",   "image/dpx" },
+      { "emf",   "image/emf" },
       { "epdf",  "application/pdf" },
       { "epi",   "application/postscript" },
       { "eps",   "application/postscript" },
@@ -1607,17 +1614,24 @@ MagickToMime(const char *magick)
       { "epsf",  "application/postscript" },
       { "ept",   "application/postscript" },
       { "fax",   "image/g3fax" },
+      { "fits",  "image/fits" },
       { "fpx",   "image/vnd.fpx" },
       { "g3",    "image/g3fax" },
       { "gif",   "image/gif" },
       { "gif87", "image/gif" },
+      { "heic",  "image/heic" },
+      { "heic-sequence", "image/heic-sequence" },
+      { "heif",  "image/heif" },
+      { "heif-sequence", "image/heif-sequence" },
       { "ico",   "image/x-icon" },
+      { "j2c",   "image/j2c" },
       { "jpeg",  "image/jpeg" },
       { "jpg",   "image/jpeg" },
+      { "jxl",   "image/jxl" },
       { "mng",   "video/x-mng" },
       { "mpeg",  "video/mpeg" },
-      { "png",   "image/png" },
       { "pdf",   "application/pdf" },
+      { "png",   "image/png" },
       { "ps",    "application/postscript" },
       { "ps2",   "application/postscript" },
       { "ps3",   "application/postscript" },
@@ -1625,7 +1639,8 @@ MagickToMime(const char *magick)
       { "tif",   "image/tiff" },
       { "tiff",  "image/tiff" },
       { "wbmp",  "image/vnd.wap.wbmp" },
-      { "webp",  "image/image/webp" }
+      { "webp",  "image/webp" },
+      { "wmf",   "image/wmf" }
     };
 
   for (i=0; i < ArraySize(MediaTypes); i++)
