@@ -38,6 +38,7 @@
 #include "magick/alpha_composite.h"
 #include "magick/color.h"
 #include "magick/monitor.h"
+#include "magick/paint.h"
 #include "magick/pixel_cache.h"
 #include "magick/pixel_iterator.h"
 #include "magick/render.h"
@@ -54,8 +55,8 @@
   {                                                                     \
     if (s >= (segment_stack+MaxStacksize))                              \
       {                                                                 \
-        MagickFreeResourceLimitedMemory(floodplane);                    \
-        MagickFreeResourceLimitedMemory(segment_stack);                 \
+        MagickFreeResourceLimitedMemory(unsigned char *,floodplane);    \
+        MagickFreeResourceLimitedMemory(SegmentInfo *,segment_stack);   \
         ThrowBinaryException2(DrawError,"SegmentStackOverflow",         \
                               image->filename);                         \
       }                                                                 \
@@ -191,8 +192,8 @@ MagickExport MagickPassFail ColorFloodfillImage(Image *image,
   if ((floodplane== (unsigned char *) NULL) ||
       (segment_stack == (SegmentInfo *) NULL))
     {
-      MagickFreeResourceLimitedMemory(floodplane);
-      MagickFreeResourceLimitedMemory(segment_stack);
+      MagickFreeResourceLimitedMemory(unsigned char *, floodplane);
+      MagickFreeResourceLimitedMemory(SegmentInfo *, segment_stack);
       ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
                             UnableToFloodfillImage);
     }
@@ -497,8 +498,8 @@ MagickExport MagickPassFail ColorFloodfillImage(Image *image,
             } /* for y ... */
         } /* if status ... */
     }
-  MagickFreeResourceLimitedMemory(segment_stack);
-  MagickFreeResourceLimitedMemory(floodplane);
+  MagickFreeResourceLimitedMemory(SegmentInfo *, segment_stack);
+  MagickFreeResourceLimitedMemory(unsigned char *, floodplane);
   return(status);
 }
 
@@ -726,7 +727,7 @@ MagickExport MagickPassFail MatteFloodfillImage(Image *image,
       start=x;
     } while (x <= x2);
   }
-  MagickFreeResourceLimitedMemory(segment_stack);
+  MagickFreeResourceLimitedMemory(SegmentInfo*, segment_stack);
   return(status);
 }
 
