@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2009-2014 GraphicsMagick Group
+% Copyright (C) 2009-2025 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -13,7 +13,6 @@
 */
 
 #include "magick/studio.h"
-#include "magick/random-private.h"
 #include "magick/random.h"
 #include "magick/semaphore.h"
 #include "magick/tsd.h"
@@ -31,6 +30,9 @@
 #if defined(USE_WIN32_THREADS)
 #  include <windows.h>
 #endif
+#if defined(MSWINDOWS) && HAVE_CRYPTGENRANDOM && HAVE_WINCRYPT_H
+#  include <wincrypt.h>
+#endif
 
 
 static MagickBool initialized = MagickFalse;
@@ -41,7 +43,7 @@ static MagickTsdKey_t kernel_key = (MagickTsdKey_t) 0;
   Acquire the default random number kernel.  Memory is owned by
   library and should not be freed.
 */
-MagickExport MagickRandomKernel* AcquireMagickRandomKernel()
+MagickExport MagickRandomKernel* AcquireMagickRandomKernel(void)
 {
   MagickRandomKernel
     *kernel;
@@ -53,11 +55,11 @@ MagickExport MagickRandomKernel* AcquireMagickRandomKernel()
   if (kernel == (MagickRandomKernel *) NULL)
     {
       kernel=MagickAllocateAlignedMemory(MagickRandomKernel *,
-					 MAGICK_CACHE_LINE_SIZE,
-					 sizeof(MagickRandomKernel));
+                                         MAGICK_CACHE_LINE_SIZE,
+                                         sizeof(MagickRandomKernel));
       if (kernel == (MagickRandomKernel *) NULL)
-	MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-			  UnableToAllocateRandomKernel);
+        MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                          UnableToAllocateRandomKernel);
       InitializeMagickRandomKernel(kernel);
       MagickTsdSetSpecific(kernel_key,(const void *) kernel);
     }
@@ -88,7 +90,7 @@ void DestroyMagickRandomGenerator()
   if (initialized)
     {
       MagickRandomKernel
-	*kernel;
+        *kernel;
 
       /* FIXME: This only frees memory associated with one thread */
       kernel=(MagickRandomKernel *) MagickTsdGetSpecific(kernel_key);
@@ -109,34 +111,59 @@ InitializeMagickRandomKernel(MagickRandomKernel *kernel)
 {
   kernel->z = 0U;
   kernel->w = 0U;
-#if HAVE_DEV_URANDOM
+#if defined(POSIX) && HAVE_DEV_URANDOM
+  /*
+    Read from /dev/urandom
+
+    MinGW's MSYS emulates /dev/random in its shell so a configure
+      test for it produces an invalid result.
+  */
   {
-    /*
-      It is not known for sure what /dev/urandom on System X will do
-      if the kernel runs out of entropy.  If it produces zeros, then
-      don't trust the result.  While Linux /dev/urandom seems to use
-      the same entropy pool as /dev/random, it apparently uses some
-      hashing algorithm on the whole entropy pool (4096 bits) so that
-      apparently random values are produced when the system runs out
-      of fresh entropy.
-    */
+    MagickBool
+      done = MagickFalse;
+
     int
       file;
 
     if ((file=open("/dev/urandom",O_RDONLY | O_BINARY)) != -1)
       {
-	if (read(file,(void *) kernel,sizeof(*kernel)) == sizeof(*kernel))
-	  {
-	    if ((kernel->z != 0U) || (kernel->w != 0U))
-              {
-                (void) close(file);
-                return;
-              }
-	  }
-	(void) close(file);
+        if (read(file,(void *) kernel,sizeof(*kernel)) == sizeof(*kernel))
+          done=MagickTrue;
+        (void) close(file);
       }
+    if (!done)
+      MagickFatalError(ResourceLimitFatalError,UnableToObtainRandomEntropy,
+                       NULL);
   }
-#endif
+#elif defined(MSWINDOWS) && HAVE_CRYPTGENRANDOM
+  /*
+    Use Windows advapi32.lib CryptGenRandom
+
+    Is claimed to be supported under Windows XP
+  */
+  {
+    MagickBool
+      done = MagickFalse;
+
+    HCRYPTPROV
+      hProvider = 0;
+
+    if (CryptAcquireContextW(&hProvider, 0, 0, PROV_RSA_FULL,
+                             CRYPT_VERIFYCONTEXT | CRYPT_SILENT))
+      {
+        /* Returns zero (FALSE) on failure */
+        if (CryptGenRandom(hProvider, (DWORD) sizeof(*kernel), (BYTE *) kernel))
+          done=MagickTrue;
+        (void) CryptReleaseContext(hProvider, 0);
+      }
+    if (!done)
+      MagickFatalError(ResourceLimitFatalError,UnableToObtainRandomEntropy,
+                       NULL);
+  }
+#else
+  /*
+    Fallback implementation
+  */
 
   /*
     Initial time of day.
@@ -163,7 +190,7 @@ InitializeMagickRandomKernel(MagickRandomKernel *kernel)
       pthread_t thread_id;
       magick_uint32_t intval;
     } pthread_union;
-    
+
     pthread_union.intval=0U;
     pthread_union.thread_id=pthread_self();
     kernel->w ^= pthread_union.intval;
@@ -178,6 +205,7 @@ InitializeMagickRandomKernel(MagickRandomKernel *kernel)
     number generator to help produce a more random seed.
   */
   kernel->w ^= (magick_uint32_t) rand();
+#endif
 }
 
 /*
