@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003-2026 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -47,7 +47,7 @@
 /*
   Define  declarations.
 */
-#define ResourceInfinity  (-1)
+/* #define MagickResourceInfinity ((magick_int64_t) (~((magick_uint64_t) 0) >> 1)) */
 #define ResourceInfoMaxIndex ((unsigned int) (sizeof(resource_info)/sizeof(resource_info[0])-1))
 
 /*
@@ -66,44 +66,48 @@ typedef enum
 typedef struct _ResourceInfo
 {
   const char
-    *name,
-    *units,
-    *env;
+    name[8],
+    units[2],
+    env[20];
 
-  magick_int64_t
+  magick_intmax_t
     value;
 
-  magick_int64_t
+  magick_intmax_t
     minimum,
-    maximum;
+    maximum,
+    highwater;
 
   LimitType
     limit_type;
 
+  SemaphoreInfo
+    *semaphore;
+
 } ResourceInfo;
 
 /*
   Global declarations.
 */
-static SemaphoreInfo
-  *resource_semaphore = (SemaphoreInfo *) NULL;
 
 /*
   Array must be in same order as ResourceType enum
 */
 #define PIXEL_LIMIT (INT_MAX/(sizeof(PixelPacket)))
-static ResourceInfo
-  resource_info[] =
+static ResourceInfo resource_info[] =
   {
-    { "",       "",  "",                    0, 0,  ResourceInfinity, SummationLimit },
-    { "disk",   "B", "MAGICK_LIMIT_DISK",   0, 0,  ResourceInfinity, SummationLimit },
-    { "files",  "",  "MAGICK_LIMIT_FILES",  0, 32, 256,              SummationLimit },
-    { "map",    "B", "MAGICK_LIMIT_MAP",    0, 0,  ResourceInfinity, SummationLimit },
-    { "memory", "B", "MAGICK_LIMIT_MEMORY", 0, 0,  ResourceInfinity, SummationLimit },
-    { "pixels", "P", "MAGICK_LIMIT_PIXELS", 0, 1,  ResourceInfinity, AbsoluteLimit  },
-    { "threads", "", "OMP_NUM_THREADS",     1, 1,  ResourceInfinity, AbsoluteLimit  },
-    { "width",  "P", "MAGICK_LIMIT_WIDTH",  0, 1,  PIXEL_LIMIT,      AbsoluteLimit  },
-    { "height", "P", "MAGICK_LIMIT_HEIGHT", 0, 1,  PIXEL_LIMIT,      AbsoluteLimit  }
+   { "",       "",  "",                    0, 0,    MagickResourceInfinity, 0, AbsoluteLimit, 0  },
+   { "disk",   "B", "MAGICK_LIMIT_DISK",   0, 0,    MagickResourceInfinity, 0, SummationLimit, 0 },
+   { "files",  "",  "MAGICK_LIMIT_FILES",  0, 32,   256,                    0, SummationLimit, 0 },
+   { "map",    "B", "MAGICK_LIMIT_MAP",    0, 0,    MagickResourceInfinity, 0, SummationLimit, 0 },
+   { "memory", "B", "MAGICK_LIMIT_MEMORY", 0, 0,    MagickResourceInfinity, 0, SummationLimit, 0 },
+   { "pixels", "P", "MAGICK_LIMIT_PIXELS", 0, 1,    MagickResourceInfinity, 0, AbsoluteLimit, 0  },
+   { "threads", "", "OMP_NUM_THREADS",     1, 1,    MagickResourceInfinity, 0, AbsoluteLimit, 0  },
+   { "width",  "P", "MAGICK_LIMIT_WIDTH",  0, 1,    PIXEL_LIMIT,            0, AbsoluteLimit, 0  },
+   { "height", "P", "MAGICK_LIMIT_HEIGHT", 0, 1,    PIXEL_LIMIT,            0, AbsoluteLimit, 0  },
+   { "read",   "B", "MAGICK_LIMIT_READ",   0, 4096, MagickResourceInfinity, 0, AbsoluteLimit, 0  },
+   { "write",  "B", "MAGICK_LIMIT_WRITE",  0, 4096, MagickResourceInfinity, 0, AbsoluteLimit, 0  },
+   { "images", "",  "MAGICK_LIMIT_IMAGES",  0, 0,   MagickResourceInfinity, 0, SummationLimit, 0 }
   };
 
 /*
@@ -117,8 +121,9 @@ static ResourceInfo
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AcquireMagickResource() acquires resources of the specified type.  True is
-%  returned if the specified resource is available otherwise False.
+%  AcquireMagickResource() acquires resources of the specified type.
+%  MagickTrue is returned if the specified resource is available otherwise
+%  MagickFalse.
 %
 %  The format of the AcquireMagickResource() method is:
 %
@@ -143,7 +148,7 @@ static ResourceInfo *GetResourceInfo(const ResourceType type)
 
   info = (ResourceInfo *) NULL;
   index = type;
-  if (index <= ResourceInfoMaxIndex)
+  if ((index != 0) && (index <= ResourceInfoMaxIndex))
     info = &resource_info[index];
 
   return info;
@@ -151,11 +156,8 @@ static ResourceInfo *GetResourceInfo(const ResourceType type)
 
 MagickExport MagickPassFail
 AcquireMagickResource(const ResourceType type,
-		      const magick_uint64_t size)
+                      const magick_uint64_t size)
 {
-  magick_uint64_t
-    value;
-
   ResourceInfo
     *info;
 
@@ -164,10 +166,11 @@ AcquireMagickResource(const ResourceType type,
 
   status=MagickPass;
 
-  LockSemaphoreInfo(resource_semaphore);
-
   if ((info=GetResourceInfo(type)))
     {
+      magick_uint64_t
+        value=0;
+
       switch(info->limit_type)
         {
         case AbsoluteLimit:
@@ -175,9 +178,19 @@ AcquireMagickResource(const ResourceType type,
             /*
               Limit depends only on the currently requested size.
             */
-            if ((info->maximum != ResourceInfinity) &&
+            value=info->value;
+            if ((info->maximum != MagickResourceInfinity) &&
                 (size > (magick_uint64_t) info->maximum))
-              status=MagickFail;
+              {
+                status=MagickFail;
+              }
+            else
+              {
+                LockSemaphoreInfo(info->semaphore);
+                if (size > (magick_uint64_t) info->highwater)
+                  info->highwater = size;
+                UnlockSemaphoreInfo(info->semaphore);
+              }
             break;
           }
         case SummationLimit:
@@ -186,25 +199,32 @@ AcquireMagickResource(const ResourceType type,
               Limit depends on sum of previous allocations as well as
               the currently requested size.
             */
+            LockSemaphoreInfo(info->semaphore);
             value=info->value+size;
-            if ((info->maximum != ResourceInfinity) &&
+            if ((info->maximum != MagickResourceInfinity) &&
                 (value > (magick_uint64_t) info->maximum))
-              status=MagickFail;
+              {
+                value=info->value;
+                status=MagickFail;
+              }
             else
-              info->value=value;
-
+              {
+                info->value=value;
+                if (value > (magick_uint64_t) info->highwater)
+                  info->highwater = value;
+              }
+            UnlockSemaphoreInfo(info->semaphore);
             break;
           }
         }
-
-      if (IsEventLogging())
+      if (IsEventLogged(ResourceEvent))
         {
           char
             f_limit[MaxTextExtent],
             f_size[MaxTextExtent],
             f_value[MaxTextExtent];
 
-          if (info->maximum == ResourceInfinity)
+          if (info->maximum == MagickResourceInfinity)
             {
               strlcpy(f_limit,"Unlimited",sizeof(f_limit));
             }
@@ -223,7 +243,7 @@ AcquireMagickResource(const ResourceType type,
             }
           else
             {
-              FormatSize(info->value, f_value);
+              FormatSize(value, f_value);
               strlcat(f_value,info->units,sizeof(f_value));
             }
 
@@ -237,7 +257,6 @@ AcquireMagickResource(const ResourceType type,
         }
     }
 
-  UnlockSemaphoreInfo(resource_semaphore);
   return(status);
 }
 
@@ -262,7 +281,76 @@ AcquireMagickResource(const ResourceType type,
 */
 MagickExport void DestroyMagickResources(void)
 {
-  DestroySemaphoreInfo(&resource_semaphore);
+  size_t
+    index;
+
+  if (IsEventLogged(ResourceEvent))
+    {
+#if 0
+      const ResourceInfo *disc_info = GetResourceInfo(DiskResource);
+      const ResourceInfo *file_info = GetResourceInfo(FileResource);
+      const ResourceInfo *map_info = GetResourceInfo(MapResource);
+      const ResourceInfo *memory_info = GetResourceInfo(MemoryResource);
+      const ResourceInfo *images_info = GetResourceInfo(ImagesResource);
+
+      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                            "Resource Consumption Summary\n"
+                            "    %c%s Limit: %" MAGICK_INTMAX_F "d, Maximum Used: %" MAGICK_INTMAX_F "d\n"
+                            "    %c%s Limit: %" MAGICK_INTMAX_F "d, Maximum Used: %" MAGICK_INTMAX_F "d\n"
+                            "    %c%s Limit: %" MAGICK_INTMAX_F "d, Maximum Used: %" MAGICK_INTMAX_F "d\n"
+                            "    %c%s Limit: %" MAGICK_INTMAX_F "d, Maximum Used: %" MAGICK_INTMAX_F "d\n"
+                            "    %c%s Limit: %" MAGICK_INTMAX_F "d, Maximum Used: %" MAGICK_INTMAX_F "d\n",
+                            toupper((int) disc_info->name[0]),disc_info->name+1, disc_info->maximum, (magick_int64_t) disc_info->highwater,
+                            toupper((int) file_info->name[0]),file_info->name+1, file_info->maximum, (magick_int64_t) file_info->highwater,
+                            toupper((int) map_info->name[0]),map_info->name+1, map_info->maximum, (magick_int64_t) map_info->highwater,
+                            toupper((int) memory_info->name[0]),memory_info->name+1, memory_info->maximum, (magick_int64_t) memory_info->highwater,
+                            toupper((int) images_info->name[0]),images_info->name+1, images_info->maximum, (magick_int64_t) images_info->highwater
+                            );
+#endif
+#if 1
+      const ResourceInfo *info;
+      for (index = 1; index < ArraySize(resource_info); index++)
+        {
+          info=&resource_info[index];
+          if (info->limit_type != SummationLimit && info->highwater == 0)
+            continue;
+          LockSemaphoreInfo(info->semaphore);
+          (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                "Resource - %c%s Limit: %" MAGICK_INTMAX_F "d,"
+                                " Maximum Used: %" MAGICK_INTMAX_F "d",
+                                toupper((int) info->name[0]),info->name+1,
+                                info->maximum, (magick_int64_t) info->highwater);
+          UnlockSemaphoreInfo(info->semaphore);
+        }
+#endif
+    }
+
+#if defined(DEBUG_MAGICK_RESOURCES) && DEBUG_MAGICK_RESOURCES
+  {
+    const ResourceInfo *info;
+
+    for (index = 1; index < ArraySize(resource_info); index++)
+      {
+        info=&resource_info[index];
+        if (info->limit_type != SummationLimit)
+          continue;
+        LockSemaphoreInfo(info->semaphore);
+        if (info->value != 0)
+          {
+            fprintf(stderr,"Resource[%s] %s! %" MAGICK_INTMAX_F "d remaining\n",
+                    info->name, info->value < 0 ? "underflow" : "leak",
+                    (magick_int64_t) info->value);
+#if defined(DEBUG_MAGICK_RESOURCES_ASSERT) && DEBUG_MAGICK_RESOURCES_ASSERT
+            assert(info->value == 0);
+#endif /* if defined(DEBUG_MAGICK_RESOURCES_ASSERT) && DEBUG_MAGICK_RESOURCES_ASSERT */
+          }
+        UnlockSemaphoreInfo(info->semaphore);
+      }
+  }
+#endif /* if defined(DEBUG_MAGICK_RESOURCES) && DEBUG_MAGICK_RESOURCES */
+
+  for (index = 1; index < ArraySize(resource_info); index++)
+    DestroySemaphoreInfo(&resource_info[index].semaphore);
 }
 
 /*
@@ -298,12 +386,14 @@ MagickExport magick_int64_t GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  LockSemaphoreInfo(resource_semaphore);
 
   if ((info=GetResourceInfo(type)))
-    resource=info->value;
+    {
+      LockSemaphoreInfo(info->semaphore);
+      resource=info->value;
+      UnlockSemaphoreInfo(info->semaphore);
+    }
 
-  UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -319,11 +409,12 @@ MagickExport magick_int64_t GetMagickResource(const ResourceType type)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetMagickResourceLimit() returns the current maximum limit for the
-%  specified resource type.
+%  specified resource type.  Returns a maximum 64-bit integer value if
+%  the resource has not been limited.
 %
 %  The format of the GetMagickResourceLimit() method is:
 %
-%      magick_uint64_t GetMagickResourceLimit(const ResourceType type)
+%      magick_int64_t GetMagickResourceLimit(const ResourceType type)
 %
 %  A description of each parameter follows:
 %
@@ -340,12 +431,14 @@ MagickExport magick_int64_t GetMagickResourceLimit(const ResourceType type)
     resource;
 
   resource=0;
-  LockSemaphoreInfo(resource_semaphore);
 
   if ((info=GetResourceInfo(type)))
-    resource=info->maximum;
+    {
+      LockSemaphoreInfo(info->semaphore);
+      resource=info->maximum;
+      UnlockSemaphoreInfo(info->semaphore);
+    }
 
-  UnlockSemaphoreInfo(resource_semaphore);
   return(resource);
 }
 
@@ -378,13 +471,19 @@ MagickExport void InitializeMagickResources(void)
     max_pixels=-1,
     max_threads=1,
     max_width=-1,
-    max_height=-1;
+    max_height=-1,
+    max_read=-1,
+    max_write=-1,
+    max_images=-1;
+
+  size_t
+    index;
 
   /*
-    Allocate semaphore.
+    Allocate semaphores.
   */
-  assert(resource_semaphore == (SemaphoreInfo *) NULL);
-  resource_semaphore=AllocateSemaphoreInfo();
+  for (index = 1; index < ArraySize(resource_info); index++)
+    resource_info[index].semaphore=AllocateSemaphoreInfo();
 
   /*
     Set Magick resource limits.
@@ -539,8 +638,8 @@ MagickExport void InitializeMagickResources(void)
   /*
     Disk, map, and memory are in units of MB and need to be scaled up.
   */
-  if (max_disk > 0)
-    max_disk  *= 1024UL*1024UL;
+  /* if (max_disk > 0) */
+  /*   max_disk  *= 1024UL*1024UL; */
   if (max_map > 0)
     max_map   *= 1024UL*1024UL;
   if (max_memory > 0)
@@ -574,15 +673,24 @@ MagickExport void InitializeMagickResources(void)
     if ((envp=getenv("MAGICK_LIMIT_HEIGHT")))
       max_height=MagickSizeStrToInt64(envp,1024);
 
+    if ((envp=getenv("MAGICK_LIMIT_READ")))
+      max_read=MagickSizeStrToInt64(envp,1024);
+
+    if ((envp=getenv("MAGICK_LIMIT_WRITE")))
+      max_write=MagickSizeStrToInt64(envp,1024);
+
+    if ((envp=getenv("MAGICK_LIMIT_IMAGES")))
+      max_images=MagickSizeStrToInt64(envp,1024);
+
 #if defined(HAVE_OPENMP)
     max_threads=omp_get_num_procs();
     (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-			  "%i CPU cores are available",(int) max_threads);
+                          "%i CPU cores are available",(int) max_threads);
     if ((envp=getenv("OMP_NUM_THREADS")))
       {
-	max_threads=MagickSizeStrToInt64(envp,1024);
-	(void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-			      "OMP_NUM_THREADS requests %i threads",(int) max_threads);
+        max_threads=MagickSizeStrToInt64(envp,1024);
+        (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                              "OMP_NUM_THREADS requests %i threads",(int) max_threads);
       }
     if (max_threads < 1)
       max_threads=1;
@@ -595,67 +703,67 @@ MagickExport void InitializeMagickResources(void)
 #  if defined(HAVE_GETRLIMIT) && defined(RLIMIT_NOFILE)
     {
       struct rlimit
-	rlimits;
+        rlimits;
 
       rlim_t
-	margin,
-	target;
+        margin,
+        target;
 
       margin=128;
       target=(rlim_t) max_files+margin;
       if (getrlimit(RLIMIT_NOFILE, &rlimits) != -1)
-	{
-	  (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-				"System file open limits are %lu soft,"
-				" %lu hard",
-				(unsigned long) rlimits.rlim_cur,
-				(unsigned long) rlimits.rlim_max);
+        {
+          (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                "System file open limits are %lu soft,"
+                                " %lu hard",
+                                (unsigned long) rlimits.rlim_cur,
+                                (unsigned long) rlimits.rlim_max);
 #    if defined(HAVE_SETRLIMIT)
-	  if (rlimits.rlim_max < target)
-	    rlimits.rlim_cur=rlimits.rlim_max;
-	  if (rlimits.rlim_cur < target)
-	    {
-	      (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-				    "Increasing file open soft limit from %lu "
-				    "to %lu",
-				    (unsigned long) rlimits.rlim_cur,
-				    (unsigned long) target);
-	      rlimits.rlim_cur=target;
-	      (void) setrlimit(RLIMIT_NOFILE, &rlimits);
-	    }
-	  if (getrlimit(RLIMIT_NOFILE, &rlimits) != -1)
-	    {
-	      if (rlimits.rlim_cur < target)
-		{
-		  if (rlimits.rlim_cur > margin*2)
-		    max_files=rlimits.rlim_cur-margin;
-		  else
-		    max_files=rlimits.rlim_cur/2;
-		}
-	    }
+          if (rlimits.rlim_max < target)
+            rlimits.rlim_cur=rlimits.rlim_max;
+          if (rlimits.rlim_cur < target)
+            {
+              (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                    "Increasing file open soft limit from %lu "
+                                    "to %lu",
+                                    (unsigned long) rlimits.rlim_cur,
+                                    (unsigned long) target);
+              rlimits.rlim_cur=target;
+              (void) setrlimit(RLIMIT_NOFILE, &rlimits);
+            }
+          if (getrlimit(RLIMIT_NOFILE, &rlimits) != -1)
+            {
+              if (rlimits.rlim_cur < target)
+                {
+                  if (rlimits.rlim_cur > margin*2)
+                    max_files=rlimits.rlim_cur-margin;
+                  else
+                    max_files=rlimits.rlim_cur/2;
+                }
+            }
 #    endif
-	}
+        }
     }
 #  elif defined(HAVE_SYSCONF) && defined(_SC_OPEN_MAX)
     {
       long
-	margin,
-	open_max,
-	target;
+        margin,
+        open_max,
+        target;
 
       margin=128;
       target=max_files+margin;
       open_max=sysconf(_SC_OPEN_MAX);
       (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-			    "System file open limit is %lu",
-			    (unsigned long) open_max);
+                            "System file open limit is %lu",
+                            (unsigned long) open_max);
       if (open_max < target)
-	{
-	  if (open_max > margin*2)
-	    max_files=open_max-margin;
-	  else
-	    max_files=open_max/2;
-	}
+        {
+          if (open_max > margin*2)
+            max_files=open_max-margin;
+          else
+            max_files=open_max/2;
+        }
     }
 #  endif
 
@@ -672,11 +780,20 @@ MagickExport void InitializeMagickResources(void)
   if (max_pixels >= 0)
     (void) SetMagickResourceLimit(PixelsResource,max_pixels);
   if (max_threads >= 0)
-    (void) SetMagickResourceLimit(ThreadsResource,max_threads);
+    {
+      (void) SetMagickResourceLimit(ThreadsResource,max_threads);
+      _UpdateMagickResourceHighwater(ThreadsResource,max_threads);
+    }
   if (max_width >= 0)
     (void) SetMagickResourceLimit(WidthResource,max_width);
   if (max_height >= 0)
     (void) SetMagickResourceLimit(HeightResource,max_height);
+  if (max_read >= 0)
+    (void) SetMagickResourceLimit(ReadResource,max_read);
+  if (max_write >= 0)
+    (void) SetMagickResourceLimit(WriteResource,max_write);
+  if (max_images >= 0)
+    (void) SetMagickResourceLimit(ImagesResource,max_images);
 }
 
 /*
@@ -711,10 +828,11 @@ MagickExport void LiberateMagickResource(const ResourceType type,
   ResourceInfo
     *info;
 
-  LockSemaphoreInfo(resource_semaphore);
-
   if ((info=GetResourceInfo(type)))
     {
+      magick_uint64_t
+        value=0;
+
       switch(info->limit_type)
         {
         case AbsoluteLimit:
@@ -722,7 +840,6 @@ MagickExport void LiberateMagickResource(const ResourceType type,
             /*
               Limit depends only on the currently requested size.
             */
-            info->value=0;
             break;
           }
         case SummationLimit:
@@ -731,20 +848,25 @@ MagickExport void LiberateMagickResource(const ResourceType type,
               Limit depends on sum of previous allocations as well as
               the currently requested size.
             */
+            LockSemaphoreInfo(info->semaphore);
             info->value-=size;
+#if defined(DEBUG_MAGICK_RESOURCES) && DEBUG_MAGICK_RESOURCES
+            assert(info->value >= info->minimum);
+#endif /* if defined(DEBUG_MAGICK_RESOURCES) && DEBUG_MAGICK_RESOURCES */
+            value=info->value;
+            UnlockSemaphoreInfo(info->semaphore);
             break;
           }
         }
 
-
-      if (IsEventLogging())
+      if (IsEventLogged(ResourceEvent))
         {
           char
             f_limit[MaxTextExtent],
             f_size[MaxTextExtent],
             f_value[MaxTextExtent];
 
-          if (info->maximum == ResourceInfinity)
+          if (info->maximum == MagickResourceInfinity)
             {
               strlcpy(f_limit,"Unlimited",sizeof(f_limit));
             }
@@ -763,7 +885,7 @@ MagickExport void LiberateMagickResource(const ResourceType type,
             }
           else
             {
-              FormatSize(info->value, f_value);
+              FormatSize(value, f_value);
               strlcat(f_value,info->units,sizeof(f_value));
             }
 
@@ -776,8 +898,6 @@ MagickExport void LiberateMagickResource(const ResourceType type,
                                 f_limit);
         }
     }
-
-  UnlockSemaphoreInfo(resource_semaphore);
 }
 
 /*
@@ -806,12 +926,13 @@ MagickExport void LiberateMagickResource(const ResourceType type,
 %
 */
 MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
-  ExceptionInfo *ARGUNUSED(exception))
+  ExceptionInfo *exception)
 {
   unsigned int
     index;
-  
-  LockSemaphoreInfo(resource_semaphore);
+
+  ARG_NOT_USED(exception);
+
   if (file == (const FILE *) NULL)
     file=stdout;
 
@@ -826,7 +947,8 @@ MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
         heading[MaxTextExtent],
         limit[MaxTextExtent];
 
-      if (resource_info[index].maximum == ResourceInfinity)
+      LockSemaphoreInfo(resource_info[index].semaphore);
+      if (resource_info[index].maximum == MagickResourceInfinity)
         {
           strlcpy(limit,"Unlimited",sizeof(limit));
         }
@@ -841,6 +963,7 @@ MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
       (void) strlcpy(environment,resource_info[index].env,sizeof(environment));
 
       fprintf(file,"%8s: %10s (%s)\n", heading, limit, environment);
+      UnlockSemaphoreInfo(resource_info[index].semaphore);
     }
   fprintf(file,
           "\n"
@@ -855,7 +978,6 @@ MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
 
   (void) fflush(file);
 
-  UnlockSemaphoreInfo(resource_semaphore);
   return(MagickPass);
 }
 /*
@@ -880,6 +1002,9 @@ MagickExport MagickPassFail ListMagickResourceInfo(FILE *file,
 %    ThreadsResource -- Threads
 %    WidthResource   -- Pixels
 %    HeightResource  -- Pixels
+%    ReadResource    -- Bytes
+%    WriteResource   -- Bytes
+%    ImagesResource  -- Discrete Images
 %
 %  The format of the SetMagickResourceLimit() method is:
 %
@@ -905,9 +1030,9 @@ MagickExport MagickPassFail SetMagickResourceLimit(const ResourceType type,
 
   status=MagickFail;
 
-  LockSemaphoreInfo(resource_semaphore);
   if ((info=GetResourceInfo(type)))
     {
+      LockSemaphoreInfo(info->semaphore);
       if (limit >= info->minimum)
         {
           char
@@ -915,10 +1040,13 @@ MagickExport MagickPassFail SetMagickResourceLimit(const ResourceType type,
 
 
           FormatSize((magick_int64_t) limit, f_limit);
-	  info->maximum = limit;
+          info->maximum = limit;
+          /* Cap highwater to new maximum */
+          if (info->maximum < info->highwater)
+            info->highwater = info->maximum;
 #if defined(HAVE_OPENMP)
-	  if (ThreadsResource == type)
-	    omp_set_num_threads((int) limit);
+          if (ThreadsResource == type)
+            omp_set_num_threads((int) limit);
 #endif /* HAVE_OPENMP */
           (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
                                 "Set %s resource limit to %s%s",
@@ -928,11 +1056,84 @@ MagickExport MagickPassFail SetMagickResourceLimit(const ResourceType type,
       else
         {
           (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
-                                "Ignored bogus request to set %s resource limit to %ld%s",
-                                info->name,(long) limit,info->units);
+                                "Ignored bogus request to set %s resource limit"
+                                " to %" MAGICK_INTMAX_F "d%s",
+                                info->name,limit,info->units);
         }
+      UnlockSemaphoreInfo(info->semaphore);
+    }
+
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   _UpdateMagickResourceHighwater                                            %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  _UpdateMagickResourceHighwater() updates the highwater for the specified
+%  resource.  The highwater value is only updated if it is greater than the
+%  minimum value for the resource, less than the current maximum for the
+%  resource, and greater than the current highwater value for the resource.
+%  The intention is that highwater values are only updated for resources
+%  successfully used rather than for resources requested.
+%
+%  The format of the _UpdateMagickResourceHighwater() method is:
+%
+%      MagickPassFail _UpdateMagickResourceHighwater(const ResourceType type,
+%        const unsigned long limit)
+%
+%  A description of each parameter follows:
+%
+%    o type: The type of resource.
+%
+%    o limit: The new proposed highwater for the resource.
+%
+%
+*/
+extern MagickPassFail _UpdateMagickResourceHighwater(const ResourceType type,
+                                                     const magick_int64_t highwater)
+{
+  ResourceInfo
+    *info;
+
+  MagickPassFail
+    status;
+
+  status=MagickFail;
+
+  if ((info=GetResourceInfo(type)))
+    {
+      LockSemaphoreInfo(info->semaphore);
+      if ((highwater > info->minimum) && (highwater <= info->maximum)
+          && (highwater > info->highwater))
+        {
+          char
+            f_highwater[MaxTextExtent];
+
+
+          FormatSize((magick_int64_t) highwater, f_highwater);
+          info->highwater = highwater;
+          (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                "Updated %s resource highwater to %s%s",
+                                info->name,f_highwater,info->units);
+          status=MagickPass;
+        }
+      else
+        {
+          (void) LogMagickEvent(ResourceEvent,GetMagickModule(),
+                                "Ignored request to set %s highwater to"
+                                " %" MAGICK_INTMAX_F "d%s",
+                                info->name,highwater,info->units);
+        }
+      UnlockSemaphoreInfo(info->semaphore);
     }
-  UnlockSemaphoreInfo(resource_semaphore);
 
   return(status);
 }
