@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 - 2013 GraphicsMagick Group
+ * Copyright (C) 2003-2025 GraphicsMagick Group
  * Copyright (C) 2003 ImageMagick Studio
  * Copyright 1991-1999 E. I. du Pont de Nemours and Company
  *
@@ -12,17 +12,44 @@
  * verify that the image is correct, only that encode/decode process
  * is repeatable.
  *
- * Written by Bob Friesenhahn <bfriesen@simple.dallas.tx.us>
+ * Written by Bob Friesenhahn <bfriesen@GraphicsMagick.org>
  *
  * The image returned by both reads must be identical (or deemed close
  * enough) in order for the test to pass.
  * */
 
+#if defined(_VISUALC_) && (_MSC_VER < 1900)
+#include <magick/studio.h>
+#endif
 #include <magick/api.h>
+#include <magick/enum_strings.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
+
+static void DescribeFrames(const ImageInfo *image_info, Image *list, const MagickBool ping)
+{
+  /* [0] AVS 70x46+072 Grayscale 8-bit adea7b1989cc5d19794a25ae3d7d0bc86f83b014f7231a869ee7b97177d54ab5 */
+  static const char descr_fmt[] = "[%s] %m %wx%h%X%y %r %q-bit %#";
+  static const char descr_fmt_ping[] = "[%s] %m %wx%h%X%y %r %q-bit";
+  Image *list_entry = list;
+
+  while (list_entry != (Image *) NULL)
+    {
+      char
+        *text;
+
+      text=TranslateText(image_info,list_entry,ping ? descr_fmt_ping : descr_fmt);
+      if (text != (char *) NULL)
+        {
+          fprintf(stdout,"%s\n", text);
+          MagickFree(text);
+        }
+      list_entry=GetNextImageInList(list_entry);
+    }
+}
 
 int main ( int argc, char **argv )
 {
@@ -67,6 +94,17 @@ int main ( int argc, char **argv )
   (void) memset(size,0,sizeof(size));
   (void) memset(filespec,0,sizeof(filespec));
 
+  /*
+    Initialize locale from environment variables (LANG, LC_CTYPE,
+    LC_NUMERIC, LC_TIME, LC_COLLATE, LC_MONETARY, LC_MESSAGES,
+    LC_ALL), but require that LC_NUMERIC use common conventions.  The
+    LC_NUMERIC variable affects the decimal point character and
+    thousands separator character for the formatted input/output
+    functions and string conversion functions.
+  */
+  (void) setlocale(LC_ALL,"");
+  (void) setlocale(LC_NUMERIC,"C");
+
   if (LocaleNCompare("rwfile",argv[0],7) == 0)
     InitializeMagick((char *) NULL);
   else
@@ -81,34 +119,14 @@ int main ( int argc, char **argv )
     {
       char
         *option = argv[arg];
-    
+
       if (*option == '-')
         {
           if (LocaleCompare("compress",option+1) == 0)
             {
               arg++;
               option=argv[arg];
-              imageInfo->compression=UndefinedCompression;
-              if (LocaleCompare("Undefined",option) == 0)
-                imageInfo->compression=UndefinedCompression;
-              if (LocaleCompare("None",option) == 0)
-                imageInfo->compression=NoCompression;
-              if (LocaleCompare("BZip",option) == 0)
-                imageInfo->compression=BZipCompression;
-              if (LocaleCompare("Fax",option) == 0)
-                imageInfo->compression=FaxCompression;
-              if (LocaleCompare("Group4",option) == 0)
-                imageInfo->compression=Group4Compression;
-              if (LocaleCompare("JPEG",option) == 0)
-                imageInfo->compression=JPEGCompression;
-              if (LocaleCompare("Lossless",option) == 0)
-                imageInfo->compression=LosslessJPEGCompression;
-              if (LocaleCompare("LZW",option) == 0)
-                imageInfo->compression=LZWCompression;
-              if (LocaleCompare("RLE",option) == 0)
-                imageInfo->compression=RLECompression;
-              if (LocaleCompare("Zip",option) == 0)
-                imageInfo->compression=ZipCompression;
+              imageInfo->compression=StringToCompressionType(option);
             }
           else if (LocaleCompare("debug",option+1) == 0)
             {
@@ -142,23 +160,27 @@ int main ( int argc, char **argv )
                   goto program_exit;
                 }
               if(imageInfo->depth != 8 && imageInfo->depth != 16 &&
-		 imageInfo->depth != 32)
+                 imageInfo->depth != 32)
                 {
                   (void) printf("-depth (%ld) not 8, 16, or 32\n",
-				imageInfo->depth);
+                                imageInfo->depth);
                   (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
                 }
             }
-	  else if (LocaleCompare("filespec",option+1) == 0)
-	    {
-	      (void) strcpy(basefilespec,argv[++arg]);
-	    }
+          else if (LocaleCompare("filespec",option+1) == 0)
+            {
+              (void) snprintf(basefilespec, sizeof(basefilespec), "%s", argv[++arg]);
+            }
           else if (LocaleCompare("log",option+1) == 0)
             {
               (void) SetLogFormat(argv[++arg]);
             }
+          else if (LocaleCompare("interlace",option+1) == 0)
+            {
+              imageInfo->interlace=StringToInterlaceType(argv[++arg]);
+            }
           else if (LocaleCompare("nocheck",option+1) == 0)
             {
               check=MagickFalse;
@@ -183,7 +205,7 @@ int main ( int argc, char **argv )
                 }
               (void) CloneString(&imageInfo->size,argv[arg]);
             }
-	  else if (LocaleCompare("stdio",option+1) == 0)
+          else if (LocaleCompare("stdio",option+1) == 0)
             {
               use_stdio=MagickTrue;
             }
@@ -202,17 +224,19 @@ int main ( int argc, char **argv )
       (void) printf("arg=%d, argc=%d\n", arg, argc);
       (void) printf ( "Usage: %s [-compress algorithm] [-debug events]"
                       " [-depth integer] [-define value] [-filespec spec]"
-                      " [-log format] [-nocheck] [-quality quality]"
-                      " [-size geometry] -stdio -verbose]"
-                      " infile format\n", argv[0] );
+                      " [-log format] [-interlace interlace] [-nocheck]"
+                      " [-quality quality] [-size geometry] [-stdio]"
+                      " [-verbose] infile format\n", argv[0] );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
-  
+
   (void) strncpy(infile, argv[arg], MaxTextExtent-1 );
+  infile[MaxTextExtent-1]='\0';
   arg++;
   (void) strncpy( format, argv[arg], MaxTextExtent-1 );
+  format[MaxTextExtent-1]='\0';
 
   magick_info=GetMagickInfo(format,&exception);
   if (magick_info == (MagickInfo *) NULL)
@@ -242,16 +266,26 @@ int main ( int argc, char **argv )
   if (use_stdio)
     {
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Reading stdio image %s", infile);
+                            "Reading stdio image %s", infile);
       imageInfo->file=fopen(infile,"rb");
+      if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for reading %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
     }
   else
     {
-      (void) strncpy( imageInfo->filename, infile, MaxTextExtent-1 );
       if (!magick_info->adjoin || !check_for_added_frames)
-	(void) strcat( imageInfo->filename, "[0]" );
+        (void) snprintf( imageInfo->filename, sizeof(imageInfo->filename), "%.*s%s",
+                         (int)(sizeof(imageInfo->filename)-sizeof("[0]")-1), infile, "[0]");
+      else
+        (void) snprintf(imageInfo->filename, sizeof(imageInfo->filename), "%s", infile);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Reading image %s", imageInfo->filename);
+                            "Reading image %s", imageInfo->filename);
     }
   original = ReadImage ( imageInfo, &exception );
   if (use_stdio)
@@ -270,7 +304,7 @@ int main ( int argc, char **argv )
   if ( original == (Image *)NULL )
     {
       (void) printf ( "Failed to read original image %s\n",
-		      imageInfo->filename );
+                      imageInfo->filename );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -286,50 +320,66 @@ int main ( int argc, char **argv )
   */
   size[0] = '\0';
   magick_info=GetMagickInfo(format,&exception);
+  if (magick_info == (MagickInfo *) NULL)
+    {
+      fprintf(stderr, "No support for \"%s\" format.\n",format);
+      exit_status = 1;
+      goto program_exit;
+    }
 
   if (magick_info->raw)
     {
       /*
-       * Specify original image size if format requires it
+       * Specify original image size (WIDTHxHEIGHT) if format requires it
        */
       FormatString( size, "%lux%lu", original->columns, original->rows );
     }
   if (IgnoreExtensionTreatment == magick_info->extension_treatment)
     {
       /*
-	Prepend magic specifier if extension will be ignored.
+        Prepend magic specifier if extension will be ignored.
       */
-      (void) strcpy(filespec,format);
-      (void) strcat(filespec,":");
-      (void) strcat(filespec,basefilespec);
+      (void) snprintf(filespec, sizeof(filespec), "%.*s:%.*s%s", 32, format,
+                      (int) sizeof(filespec)-37, basefilespec, ".%s");
     }
   else
     {
-      strcpy(filespec,basefilespec);
+      (void) snprintf(filespec, sizeof(filespec), "%.*s%s",
+                      (int) (sizeof(filespec)-sizeof(".%s")), basefilespec,
+                      ".%s");
     }
 
-  (void) strcat(filespec,".%s");
-  (void) sprintf( filename, filespec, 1, format );
+  (void) snprintf( filename, sizeof(filename), filespec, 1, format );
   (void) remove(filename);
 
   /*
    * Save image to file
    */
-  (void) strncpy( original->magick, format, MaxTextExtent-1 );
+  (void) strncpy( original->magick, format, MaxTextExtent );
+  original->magick[MaxTextExtent-1]='\0';
   (void) fflush(stdout);
 
   strcpy(imageInfo->filename,"");
   if (use_stdio)
     {
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Writing stdio image %s", filename);
+                            "Writing stdio image %s", filename);
       imageInfo->file=fopen(filename,"wb+");
+      if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for writing %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
     }
   else
     {
-      (void) strncpy( original->filename, filename, MaxTextExtent-1 );
+      (void) strncpy( original->filename, filename, MaxTextExtent );
+      original->filename[MaxTextExtent-1]='\0';
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Writing image %s", original->filename);
+                            "Writing image %s", original->filename);
     }
   original->delay = 10;
   if (!WriteImage ( imageInfo, original ))
@@ -360,14 +410,24 @@ int main ( int argc, char **argv )
       ping_error = MagickFalse;
 
     strcpy(imageInfo->filename,"");
-    (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
+    (void) strncpy( imageInfo->magick, format, MaxTextExtent );
+    imageInfo->magick[MaxTextExtent-1]='\0';
     if (use_stdio)
       {
-	imageInfo->file=fopen(filename,"rb");
+        imageInfo->file=fopen(filename,"rb");
+        if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for reading %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
       }
     else
       {
-	strncpy( imageInfo->filename, filename, MaxTextExtent-1 );
+        strncpy( imageInfo->filename, filename, MaxTextExtent );
+        imageInfo->filename[MaxTextExtent-1]='\0';
       }
     if ( size[0] != '\0' )
       (void) CloneString( &imageInfo->size, size );
@@ -375,49 +435,59 @@ int main ( int argc, char **argv )
     ping_image = PingImage(imageInfo, &exception );
     if (use_stdio)
       {
-	(void) fclose(imageInfo->file);
-	imageInfo->file = (FILE *) NULL;
+        (void) fclose(imageInfo->file);
+        imageInfo->file = (FILE *) NULL;
       }
     if (exception.severity != UndefinedException)
       {
         if (exception.reason)
           (void) printf("    reason:%s\n",exception.reason);
-	CatchException(&exception);
-	(void) fflush(stderr);
-	ping_error = MagickTrue;
+        CatchException(&exception);
+        (void) fflush(stderr);
+        ping_error = MagickTrue;
       }
     if ( ping_image == (Image *)NULL )
       {
-	(void) printf ( "Failed to ping image from file \"%s\" in format %s\n",
-			filename, imageInfo->magick );
-	(void) fflush(stdout);
-	ping_error = MagickTrue;
+        (void) printf ( "Failed to ping image from file \"%s\" in format %s\n",
+                        filename, imageInfo->magick );
+        (void) fflush(stdout);
+        ping_error = MagickTrue;
       }
     else
       {
-	/* Print a short description of the image to stdout */
-	DescribeImage( ping_image, stdout, MagickFalse );
-	DestroyImageList( ping_image );
+        /* Print a short description of the image to stdout */
+        /* DescribeFrames(imageInfo, ping_image, MagickTrue); */
+        DestroyImageList( ping_image );
       }
     if (ping_error)
       {
-	exit_status = 1;
-	goto program_exit;
+        exit_status = 1;
+        goto program_exit;
       }
   }
 
   /*
    * Read image back from file
    */
-  (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
+  (void) strncpy( imageInfo->magick, format, MaxTextExtent );
+  imageInfo->magick[MaxTextExtent-1]='\0';
   strcpy(imageInfo->filename,"");
   if (use_stdio)
     {
       imageInfo->file=fopen(filename,"rb");
+      if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for reading %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
     }
   else
     {
-      strncpy( imageInfo->filename, filename, MaxTextExtent-1 );
+      strncpy( imageInfo->filename, filename, MaxTextExtent );
+      imageInfo->filename[MaxTextExtent-1]='\0';
     }
   if ( size[0] != '\0' )
     (void) CloneString( &imageInfo->size, size );
@@ -452,19 +522,29 @@ int main ( int argc, char **argv )
   (void) remove(filename);
 
   (void) sprintf( filename, filespec, 2, format );
-  (void) strncpy( original->magick, format, MaxTextExtent-1 );
+  (void) strncpy( original->magick, format, MaxTextExtent );
+  original->magick[MaxTextExtent-1]='\0';
   strcpy(imageInfo->filename,"");
   if (use_stdio)
     {
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Writing stdio image %s", filename);
+                            "Writing stdio image %s", filename);
       imageInfo->file=fopen(filename,"wb+");
+      if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for writing %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
     }
   else
     {
-      (void) strncpy( original->filename, filename, MaxTextExtent-1 );
+      (void) strncpy( original->filename, filename, MaxTextExtent );
+      original->filename[MaxTextExtent-1]='\0';
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Writing stdio image %s", original->filename);
+                            "Writing stdio image %s", original->filename);
     }
   original->delay = 10;
   (void) fflush(stdout);
@@ -485,15 +565,25 @@ int main ( int argc, char **argv )
   /*
    * Read image back from file
    */
-  (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
+  (void) strncpy( imageInfo->magick, format, MaxTextExtent );
+  imageInfo->magick[MaxTextExtent-1]='\0';
   strcpy(imageInfo->filename,"");
   if (use_stdio)
     {
       imageInfo->file=fopen(filename,"rb");
+      if (imageInfo->file == (FILE *) NULL)
+        {
+          (void) printf ( "Failed to open stdio for reading %s\n",
+                          imageInfo->filename );
+          (void) fflush(stdout);
+          exit_status = 1;
+          goto program_exit;
+        }
     }
   else
     {
-      (void) strncpy( imageInfo->filename, filename, MaxTextExtent-1 );
+      (void) strncpy( imageInfo->filename, filename, MaxTextExtent );
+      imageInfo->filename[MaxTextExtent-1]='\0';
     }
   if ( size[0] != '\0' )
     (void) CloneString( &imageInfo->size, size );
@@ -518,16 +608,20 @@ int main ( int argc, char **argv )
       goto program_exit;
     }
 
+  /* Print a short description of the image to stdout */
+  DescribeFrames(imageInfo, final, MagickFalse);
+  (void) fflush(stdout);
+
   if (check)
     {
       /*
        * Check final output
        */
       double
-	fuzz_factor = 0;
+        fuzz_factor = 0;
 
       /*
-	Some formats are lossy.
+        Some formats are lossy.
       */
       if ((!strcmp( "CIN", format ) && (QuantumDepth == 8)) ||
           (!strcmp( "CMYK", format )) ||
@@ -537,85 +631,111 @@ int main ( int argc, char **argv )
           (!strcmp( "JPEG", format )) ||
           (!strcmp( "JPG", format )) ||
           (!strcmp( "JPG24", format )) ||
+          (!strcmp( "JXL", format )) ||
           (!strcmp( "PAL", format )) ||
           (!strcmp( "PCD", format )) ||
           (!strcmp( "PCDS", format )) ||
           (!strcmp( "UYVY", format )) ||
           (!strcmp( "WEBP", format )) ||
           (!strcmp( "YUV", format )) ||
+
+          (!strcmp( "EPDF", format ))  ||
+          (!strcmp( "EPI", format ))  ||
+          (!strcmp( "EPS", format ))  ||
+          (!strcmp( "EPSF", format ))  ||
+          (!strcmp( "EPSI", format ))  ||
+          (!strcmp( "EPT", format ))  ||
+          (!strcmp( "PDF", format ))  ||
+          (!strcmp( "PS", format ))  ||
+          (!strcmp( "PS2", format ))  ||
+
           (final->compression == JPEGCompression))
         fuzz_factor = 0.06;
 
+      switch(imageInfo->compression)
+        {
+        case JPEGCompression:
+        case JPEG2000Compression:
+        case WebPCompression:
+          {
+            fuzz_factor = 0.06;
+            break;
+          }
+        default:
+          {
+          }
+        }
+
       {
-	Image
-	  *o,
-	  *f;
-	
-	unsigned long
-	  frame=0;
+        Image
+          *o,
+          *f;
 
-	/*
-	  Verify that frame pixels are identical (or close enough).
-	*/
-	for (o=original, f=final, frame=0;
-	     ((o != (Image *) NULL) && (f != (Image *) NULL)) ;
-	     o = o->next, f = f->next, frame++)
-	  {
-	    printf("Checking frame %ld...\n",frame);
-	    if ( !IsImagesEqual(o, f ) &&
-		 (original->error.normalized_mean_error > fuzz_factor) )
-	      {
-		(void) printf( "R/W file check for format \"%s\" failed "
-			       "(frame = %ld): %.6f/%.6f/%.6fe\n",
-			       format,frame,
-			       original->error.mean_error_per_pixel,
-			       original->error.normalized_mean_error,
-			       original->error.normalized_maximum_error);
-		(void) fflush(stdout);
-		exit_status = 1;
-		goto program_exit;
-	      }
-	  }
- 
-	if (check_for_added_frames)
-	  {
-	    /*
-	      Verify that reads from file R/W #1 and file R/W #2 did
-	      return the same number of frames.
-	    */
-	    if ((o != (Image *) NULL) && (f != (Image *) NULL))
-	      {
-		(void) printf("R/W file check for format \"%s\" failed due to "
-			      "differing number of returned frames (%ld vs %ld)\n",
-			      format,
-			      GetImageListLength(original),
-			      GetImageListLength(final));
-		exit_status = 1;
-		goto program_exit;
-	      }
+        unsigned long
+          frame=0;
 
-	    /*
-	      If format supports multiple frames, then we should expect
-	      that frames are not lost (or spuriously added) due to
-	      read/write of format.
-	    */
-	    if (magick_info->adjoin)
-	      {
-		unsigned long
-		  final_frames;
+        /*
+          Verify that frame pixels are identical (or close enough).
+        */
+        for (o=original, f=final, frame=0;
+             ((o != (Image *) NULL) && (f != (Image *) NULL)) ;
+             o = o->next, f = f->next, frame++)
+          {
+            printf("Checking frame %ld...\n",frame);
+            if ( !IsImagesEqual(o, f ) &&
+                 (original->error.normalized_mean_error > fuzz_factor) )
+              {
+                (void) printf( "R/W file check for format \"%s\" failed "
+                               "(frame = %ld): %.6f/%.6f/%.6fe\n",
+                               format,frame,
+                               original->error.mean_error_per_pixel,
+                               original->error.normalized_mean_error,
+                               original->error.normalized_maximum_error);
+                (void) fflush(stdout);
+                exit_status = 1;
+                goto program_exit;
+              }
+          }
 
-		final_frames=GetImageListLength(final);
-		if (original_frames != final_frames)
-		  {
-		    (void) printf("R/W file check for format \"%s\" failed due "
-				  "to differing number of returned frames (%ld "
-				  "vs %ld) from original file\n",
-				  format,original_frames,final_frames);
-		    exit_status = 1;
-		    goto program_exit;
-		  }
-	      }
-	  }
+        if (check_for_added_frames)
+          {
+            /*
+              Verify that reads from file R/W #1 and file R/W #2 did
+              return the same number of frames.
+            */
+            if ((o != (Image *) NULL) && (f != (Image *) NULL))
+              {
+                (void) printf("R/W file check for format \"%s\" failed due to "
+                              "differing number of returned frames (%ld vs %ld)\n",
+                              format,
+                              GetImageListLength(original),
+                              GetImageListLength(final));
+                exit_status = 1;
+                goto program_exit;
+              }
+
+            /*
+              If format supports multiple frames, then we should expect
+              that frames are not lost (or spuriously added) due to
+              read/write of format.
+            */
+            if (magick_info->adjoin)
+              {
+                unsigned long
+                  final_frames;
+
+                final_frames=GetImageListLength(final);
+                if (original_frames != final_frames)
+                  {
+                    (void) printf("R/W file check for format \"%s\" failed due "
+                                  "to differing number of returned frames (%ld "
+                                  "vs %ld) from original file\n",
+                                  format,original_frames,final_frames);
+                    exit_status = 1;
+                    goto program_exit;
+                  }
+              }
+          }
       }
     }
 
